/******************************************************************************/
/*  %W% %D% %T%  */
/*                                                                            */
/*                                                                            */
/*                                                                            */
/*  Pascal-Programm:     Vera Kell                 Aachen, 31.07.1989         */
/*                                                                            */
/*  ANSI-C  Version:     Raphael M. Grochtmann     Aachen, 31.08.1990         */
/*                                                                            */
/*                                                                            */
/******************************************************************************/



#include <stdio.h>

#include "ext.h"
#include "main.h"
#include "list.h"
#include "graph.h"
#include "xgraphics.h"
#include "levgen.h"
#include "levper.h"
#include "finetune.h"
#include "draw.h"



typedef boolean matrixboolarray[NMAX+1];



/* globale Variablen */
boolean			printer;				/* wird noch in graph.c verwendet */
FILE			*druck_fp;				/* druck-filepointer, wird noch in graph.c verwendet */
char			full_filename[300];     /* wird noch in main.c und transamore.c verwendet */



/* lokale Variablen */
static int		choice;
static ptupel	aut;
static char		printfile[300];


static void replace(strg strin)
{
 int i=0;
 while(strin[i]!='\0')
   {if(strin[i]==epsletter)strin[i]='@';i++;}
}


static void BuildStopStateArray(ptupel a, matrixboolarray *stoparray)
{
	int			j;
	pinteger	p;

	for (j = 1; j <= NMAX; j++)
		(*stoparray)[j] = FALSE;
	p = a.qe;
	while (p != NULL) {
		(*stoparray)[p->pi] = TRUE;
		p = p->psucc;
	}
} /* BuildStopStateArray */



static void sstr(int i, strg st)
{
	int		ihelp, rest, j, count;
	char	c;

	ihelp = abs(i);
	count = 0;
	if (ihelp == 0) {
		st[0] = '0';
		st[1] = '\0';
	} else {
		while (ihelp > 0) {
			rest = ihelp % 10;
			st[count++] = '0' + rest;
			ihelp -= rest;
			ihelp /= 10;
		}
		if (i < 0)
			st[count] = '-';
		count--;
		for (j=0; j < (count + 1) / 2; j++) {
			c = st[j];
			st[j] = st[count - j];
			st[count - j] = c;
		}
		if (count < STRLENGTH - 1)
			st[count + 1] = '\0';
	}
} /*sstr*/



static void DrawFiniteAutomata(ptupel a, plevel levlist)
{
	int				state, statesuc;
	int				x, y, x1, y1, x2, y2, curvrad;
	matrixboolarray stopstate;
	strg			statest, symbst;
	pvertex			qlist;
	plist			suclist, levllist, prelist;

	BuildStopStateArray(a, &stopstate);
	while (levlist != NULL) {
		qlist = levlist->qlist;
		while (qlist != NULL) {
			state = qlist->q;
			x = qlist->xkoor;
			y = qlist->ykoor;
                        if(!withoutlaststate || (state != laststate))
                          {if (state <= a.q  &&  x != 0  &&  y != 0) 
                             {sstr(state-1, statest);
			      DrawState(x, y, STATELEN, statest);
			     }
			   if (state == a.qa)
				DrawBeginArrow(x, y, ARREDLEN, ARROWLEN);
                          }
			if (stopstate[state] == TRUE)
				DrawCircle(x, y, STATELEN + STSTOFFS);
			suclist = qlist->suc;
			while(suclist != NULL) 
                             {statesuc = suclist->edge->q;
			      x1 = suclist->edge->xkoor;
			      y1 = suclist->edge->ykoor;
			      strcpy(&symbst, (char *)(suclist->edgest)); /* oma, 3.1.95 */
                              if(epstrans) /* replace a letter by @ */
                                replace(symbst);
                              if(withoutlaststate &&(state == laststate))
                                 DrawBeginArrow(x1, y1, ARREDLEN, ARROWLEN);
                              else
                                {
                                 if(suclist->edgeint == 1)
			  	   DrawDoubleRedArrow(x, y, x1, y1, ARREDLEN, ARROWLEN, symbst);
				 else 
                                   {if(statesuc > a.q) 
                                      {DrawRedLine(x, y, x1, y1, ARREDLEN, symbst);
				       x2 = suclist->edge->suc->edge->xkoor;
				       y2 = suclist->edge->suc->edge->ykoor;
				       DrawRoundVertex(x, y, x1, y1, x2, y2, ARREDLEN);
				      } 
                                    else
				      DrawRedArrow(x, y, x1, y1, ARREDLEN, ARROWLEN, symbst);
				   }
			        }
			      suclist = suclist->listsucc;
                             }
			levllist = qlist->levl;
			while (levllist != NULL) {
				x1 = levllist->edge->xkoor;
				y1 = levllist->edge->ykoor;
				strcpy(&symbst, levllist->edgest);
                                if(epstrans) /* replace a letter by @ */
                                    replace(symbst);
				curvrad = CURVCONST + abs(levllist->edgeint) * 8;
				DrawCurvedArrow(x, y, x1, y1, curvrad, ARREDLEN, ARROWLEN, symbst);
				levllist = levllist->listsucc;
			}
			if (qlist->barryleft == 1) {
				prelist = qlist->pre;
				while (prelist != NULL  &&  prelist->edgest[0] == '\0')
					prelist = prelist->listsucc;
				if (prelist != NULL)
					DrawLoop(x, y, prelist->edgeint, ARREDLEN, prelist->edgest);
			}
			qlist = qlist->vertsucc;
		}
		levlist = levlist->levsucc;
	}
} /* DrawFiniteAutomata */



static void GetHPFname(strg *zname, char *qname)
{
	int i, last;

	i = last = 0;
	while (qname[i] != '\0') {
		if (qname[i] == '/')
			last  =  i;
		i++;
	}
	last++;
	strcpy(zname, &qname[last]);
} /* GetHPFname */



static void DisplayAutomata(ptupel *a, int choice)
{
	int				statenr, length, width, i, j;
	boolean			symm, dumbreak;
	matrixlistarray	matrix;
	plevel			levlist;
	int				xfile, yfile;
	strg			hpfname, ausgabe;
	double			xr, yr;

	levlist = NULL;
	symm = dumbreak = FALSE;
	AutTabTransform(*a, matrix);
	if ((*a).q < MAXDISPLAYED) {	/* nur Automaten mit < MAXDISPLAYED Zustaenden zeichnen */
		switch (choice) {
		case DIRSYM:
			AddIDummyLev(*a, matrix, &levlist, &statenr, &length, &width, &dumbreak);
			if (!dumbreak) {
				Barryalg(levlist);
				PriorityAlg(SYMM, levlist, (*a).q, width);
			}
			symm = TRUE;
			break;
		case DIRNOSYM:
			AddIDummyLev(*a, matrix, &levlist, &statenr, &length, &width, &dumbreak);
			if (!dumbreak) {
				Barryalg(levlist);
				PriorityAlg(NOSYMM, levlist, (*a).q, width);
			}
			break;
		case NODIRSYM:
			AddIIDummyLev(*a, matrix, &levlist, &statenr, &length, &width);
			Barryalg(levlist);
			PriorityAlg(SYMM, levlist, (*a).q, width);
			symm = TRUE;
			break;
		case NODIRNOSYM:
			AddIIDummyLev(*a, matrix, &levlist, &statenr, &length, &width);
			Barryalg(levlist);
			PriorityAlg(NOSYMM, levlist, (*a).q, width);
			break;
		}
	} else {
		xr = (double)XGRMAX / 2.0 - (double)XGRMAX / 20.0;
		if (printer) {
			strcpy(&ausgabe, "no correct mode selected");
			yr = (double)YGRMAX / 2.0;
		} else {
			strcpy(&ausgabe, "mode not available");
			yr  =  (double)YGRMAX / 2.0;
		}
		CText(round(xr), round(yr), ausgabe);
		dumbreak = TRUE;
	}
	if (!dumbreak) {
		BuildAutomata(*a, levlist, matrix, symm);
		if (!printer) {
			DrawFiniteAutomata(*a, levlist);
		} else {
			if ((druck_fp = fopen(printfile, "w")) == NULL) {
				fprintf(stderr, "*** ERROR: can`t open %s ***\n", printfile);
				exit(1);
			}
			fprintf(druck_fp, "IN;\n");
			fprintf(druck_fp, "IP254,600,11254,8380;SC0,2304,0,1664;\n");
			fprintf(druck_fp, "SI0.15,0.255;\n");	/*0,2880,0,2080*/
			xfile = yfile = 20;
			GetHPFname(&hpfname, full_filename);
			DrawSymbol(xfile, yfile, hpfname);
			DrawFiniteAutomata(*a, levlist);
			fprintf(druck_fp, "AF;\n");
		}
		for (i=1; i <= statenr; i++)
			for (j=1; j <= (*a).ssize; j++)
				MatListDel(&matrix[i][j]);
		LevelListDel(&levlist);
	}
} /* DisplayAutomata */



void InitDFA(void)
{
	aut.qe = NULL;
	aut.delta = NULL;
	ChangeAmoreFile(&aut);
} /* InitDFA */



void DoDFA (int choice, int prn)
{
	if (prn == 0)
		printer = FALSE;
	else {
		printer = TRUE;
		strcpy(printfile, full_filename);
		switch (choice) { 
			case 1: strcat(printfile, ".ds.hp"); break;
			case 2: strcat(printfile, ".da.hp"); break;
			case 3: strcat(printfile, ".us.hp"); break;
			case 4: strcat(printfile, ".ua.hp"); break;
		}
	}
	DisplayAutomata(&aut, choice);
} /* DoDFA */



void get_full_filename(filename)
	char *filename;
{
	char *dir;
	char *getenv();

	full_filename[0] = '\0';
	dir = getenv("DISPLAYDIR");
	if (dir != NULL)
		strcpy(full_filename, dir);
	else {
		dir = getenv("PWD");
		strcpy(full_filename, dir);
	}
	strcat(full_filename, "/");
	strcat(full_filename, filename);
} /*get_full_filename*/
