/******************************************************************************/
/*  %W% %D% %T%  */
/*                                                                            */
/*                                                                            */
/*                                                                            */
/*  Pascal-Programm:     Vera Kell                 Aachen, 31.07.1989         */
/*                                                                            */
/*  ANSI-C  Version:     Raphael M. Grochtmann     Aachen, 31.08.1990         */
/*                                                                            */
/*                                                                            */
/* ( Aufbau des Automatengraphen anhand einer Graphliste )                    */
/******************************************************************************/



#include <stdio.h>

#include "ext.h"
#include "main.h"
#include "list.h"
#include "graph.h"



typedef int     matrixadjarray[NMAX+1][NMAX+1];
typedef double  matrixrealarray[NMAX+1];



static void DrLevListPosSort(plevel levlist)
	/* Sortiert Zustandsliste anhand der Positionen der Zustaende und loescht */
	/* Vorgaenger, Nachfolger und Ebenenkantenliste */
{
	int			j, fstelem,	posmax;
	pvertex		qlist, qarray[NMAX];
	plevel		levhelp;

	levhelp = levlist;
	while (levhelp != NULL) {
		for (j=1; j <= NMAX; j++)
			qarray[j] = NULL;
		qlist = levhelp->qlist;
		posmax = 1;
		while (qlist != NULL) {
			qarray[qlist->pos] = qlist;
			if (posmax < qlist->pos)
				posmax = qlist->pos;
			PListDel(&qlist->pre);
			qlist->pre = NULL;
			PListDel(&qlist->levl);
			qlist->levl = NULL;
			PListDel(&qlist->suc);
			qlist->suc = NULL;
			qlist = qlist->vertsucc;
		}
		fstelem = 1;
		while (qarray[fstelem] == NULL)
			fstelem++;
		qlist = levhelp->qlist = qarray[fstelem++];
		for (j=fstelem; j <= posmax; j++)
			if (qarray[j] != NULL) {
				qlist->vertsucc = qarray[j];
				qlist = qlist->vertsucc;
			}
		qlist->vertsucc = NULL;
		levhelp = levhelp->levsucc;
	}
} /*DrLevListPosSort*/



static void TestDoubleTrans(int q1, int q2, int ssize, matrixlistarray matrix, boolean *doublearrow)
	/* matrix ist NICHT Var-Parameter (daher lokale Kopie, s.u.) */
	/* Prueft anhand der Transitionstabelle, ob es sich um eine Doppelkante */
	/* handelt. */
{
	pmatrix		help;
	int			j;

	*doublearrow = FALSE;  /* Test, ob es sich um einen Doppelpfeil handelt */
	for (j=1; j <= ssize; j++) {
		help = matrix[q2][j];		/* Kopie des Matixelements */
		while (help != NULL  &&  *doublearrow == FALSE) {
			if (help->pmi == q1)
				*doublearrow = TRUE;
			help = help->pmsucc;
		}
	}
} /* TestDoubleTrans */



static void AddTransition(ptupel a, plevel levlist, matrixlistarray matrix)
	/* matrix ist var-Parameter */
	/* Gerichtete Kanten mit Beschriftung werden in die Graphliste eingefuegt */
	/* (z.B. Doppel-, Schleifen-, Ebenen- und einfache Kanten). */
{
	struct statrec {
		pvertex vert;
		int		lev;
	};
	int				j, q1, q2, doubl;
	boolean			doubletrans, leveledge;
	strg			levst, st;
	pvertex			qlist, vertbeg, vertend;
	plevel			levhelp;
	pmatrix			matelem;
	struct statrec	qarray[NMAX+1];

	levst[0] = 'l';					/* l entspricht Level */
	levhelp = levlist;
	while (levhelp != NULL) {		/* Initialisierung des Feldes qarray[] */
		qlist = levhelp->qlist;
		while (qlist != NULL) {
			qarray[qlist->q].vert = qlist;			/* qarray[i].vert zeigt auf Knoten i */
			qarray[qlist->q].lev = levhelp->lev;	/* Ebene, in der Knoten i auftritt */
			qlist->barryleft = 0;
			qlist->barryright = 0;
			qlist = qlist->vertsucc;
		}
		levhelp = levhelp->levsucc;
	}
	levhelp = levlist;
	while (levhelp != NULL) {		/* Zuordnen der gerichteten Kanten */
		qlist = levhelp->qlist;
		while (qlist != NULL) {
			q1 = qlist->q;
			for (j=1; j <= a.ssize; j++) {
				matelem = matrix[q1][j];
				while (matelem != NULL) {
					q2 = matelem->pmi;
					if (matelem->pmstrg[0] != '\0'  ||  q1 > a.q  ||  q2 > a.q) { 
						if (q1 == q2) {
							PListAddBeg(&qlist->pre, 0, matelem->pmstrg, qlist);
							qlist->barryleft = 1;
						} else {
							leveledge = FALSE;
							if (qarray[q2].lev == levhelp->lev) {
								leveledge = TRUE;
								if (qlist->pos < qarray[q2].vert->pos) {
									vertbeg = qlist;
									vertend = qarray[q2].vert;
								} else {
									vertbeg = qarray[q2].vert;
									vertend = qlist;
								}
								vertbeg = vertbeg->vertsucc;
								while (vertbeg->q > a.q)
									vertbeg = vertbeg->vertsucc;
								if (vertbeg == vertend)
									leveledge = FALSE;
							}
							if (leveledge == TRUE) {
								PListAddBeg(&qlist->levl, 0, matelem->pmstrg, qarray[q2].vert);
								PListAddBeg(&qarray[q2].vert->pre, 0, levst, qlist);
							} else {
								TestDoubleTrans(q1, q2, a.ssize, matrix, &doubletrans);
								doubl = (doubletrans) ?  1 :  0;
								PListAddBeg(&qlist->suc, doubl, matelem->pmstrg, qarray[q2].vert);
								st[0] = '\0';
								PListAddBeg(&qarray[q2].vert->pre, 0, st, qlist);
							}
						}
					}
					matelem = matelem->pmsucc;
				}
			}
			qlist = qlist->vertsucc;
		}
		levhelp = levhelp->levsucc;
	}
} /*AddTransition*/



static void BuildCoverMat(pvertex qlist, matrixadjarray covmat, matrixlevelarray qarray, int qnr)
	/* covmat, qarray sind var-Parameter */
	/* Aufbau der Adjazenzmatrix covmat[.,.] fuer Ebenenkanten */
{
	int				i, j, q2;
	matrix1array	qposar;
	plist			levllist;

	for (i=1; i <= qnr; i++) {		/* Initialisierung: qarray[.]=NULL; covmat[.,.]=0 */
		qarray[i] = NULL;
		for (j=1; j <= qnr; j++)
			covmat[i][j] = 0;
	}
	j = 1;
	while (qlist != NULL) {
		qarray[j] = qlist;			/* qarray[.]->Zustand der akt. Liste qlist */
		qposar[qlist->q] = j;		/* qposar[.]->Index des Zust. q in qarray[ind] */
		qlist = qlist->vertsucc;
		j++;
	}
	for (j=1; j <= qnr; j++) {		/* gekruemmte Ebenenkanten in covmat[.,.] eintragen */
		levllist = qarray[j]->levl;
		while (levllist != NULL) {
			q2 = levllist->edge->q;
			if (abs(qposar[q2]-j) > 1)
				covmat[j][qposar[q2]] = 1;
			levllist = levllist->listsucc;
		}
	}
} /* BuildCoverMat */



static void FindFreeCover(pcover covlist, int *cov, int edbeg, int edend, int qnr)
{
	int			edhelp, j, covnr;
	boolean		found;
	pcover		covhelp, covpre;

	found = FALSE;
	if (edbeg > edend) {
		edhelp = edbeg;
		edbeg = edend;
		edend = edhelp;
	}
	covhelp = covlist;
	while (covhelp != NULL  &&  !found) {			/* kl. Schale fuer Eb.kante suchen */
		if (covhelp->covfree >= edbeg - edend) {
			found = TRUE;							/* found=TRUE, falls Schale gefunden */
			for (j=edbeg; j <= edend -1; j++)
				if (covhelp->covar[j] != 0)
					found = FALSE;
		}
		covpre = covhelp;
		covhelp = covhelp->covsucc;
	}
	if (found == FALSE) {							/* neue Schale, falls in vorh. schon belegt */
		covnr = covpre->covnr + 1;
		CoverAddEnd(&covpre, covnr, qnr-1);
	}
	*cov = covpre->covnr;							/* Schalennummer fuer Ebenenkante */
	for (j=edbeg; j <= edend-1; j++)
		covpre->covar[j] = 1;						/* covlist akt. */
	covpre->covfree = covpre->covfree - abs(edbeg - edend);
} /* FindFreeCover */



static void AddCoverRadius(pvertex qlevbeg, pvertex qlevend, int qcov)
	/* Eintrag der ermittelten Schalennummer in die Ebenenliste */
{
	plist		levllist;

	levllist = qlevbeg->levl;
	while (levllist != NULL  &&  levllist->edge != qlevend) {
		levllist = levllist->listsucc;			/* Suche akt. Ebenenkante */
	}
	if (levllist != NULL)
		levllist->edgeint = qcov;		/* Schalennummer eintragen */
} /* AddCoverRadius */



static void FindCoverRadius(matrixlevelarray qarray, matrixadjarray covmat, int qnr)
	/* qarray ist var-Parameter */
{
	int		line, col, colhelp, covleft, covright;
	pcover	covlistleft, covlistright;

	CoverBeg(&covlistleft, 1, qnr - 1);
	CoverBeg(&covlistright, 1, qnr - 1);
	colhelp = 3;
	while (colhelp <= qnr) {		/* Durchlaufe covmat[,] diagonal, {ne mit Spalte (Zeile) 3 */
		line = 1;
		col = colhelp;
		while (col <= qnr) {
			if (covmat[line][col] != 0) {			/* Ebenenkante von oben nach unten */
				FindFreeCover(covlistleft, &covleft, line, col, qnr);		/* kl. freie Schale */
				AddCoverRadius(qarray[line], qarray[col], -covleft);		/* in Liste eintragen */
			}
			if (covmat[col][line] != 0) {			/* Ebenenkante von unten nach oben */
				FindFreeCover(covlistright, &covright, col, line, qnr);		/* kl. freie Schale */
				AddCoverRadius(qarray[col], qarray[line], covright);		/* in Liste eintragen */
			}
			col++;
			line++;
		}
		colhelp++;
	}
	CoverDel(&covlistleft);
	CoverDel(&covlistright);
} /* FindCoverRadius */



static void FindLevelEdgeRadius(plevel levlist)
{
	plevel				levhelp;
	pvertex				qlist;
	matrixlevelarray	qarray;
	matrixadjarray		covmat;

	levhelp = levlist;
	while (levhelp != NULL) {
		if (levhelp->qnumber > 2) {
			qlist = levhelp->qlist;
			BuildCoverMat(qlist, covmat, qarray, levhelp->qnumber);
			FindCoverRadius(qarray, covmat, levhelp->qnumber);
		}
		levhelp = levhelp->levsucc;
	}
} /* FindLevelEdgeRadius */



static void GraphKoord (plevel levlist)
{
	int			width, length, qnr, qebene, qpos, q, MaxLenG, MaxWidG;
	double		widdist, lendist;
	plevel		levhelp;
	pvertex		qliste;

	MaxLenG = XGRMAX - 2 * DIST;
	MaxWidG = YGRMAX - 2 * DIST;
	width = length = 0;
	levhelp = levlist;
	while (levhelp != NULL) {
		length++;
		if (levhelp->qnumber > width)
			width = levhelp->qnumber;
		levhelp = levhelp->levsucc;
	}
	lendist = (double)MaxLenG / (double)(length + 1);
	widdist = (double)MaxWidG / (double)(width + 1);
	while (levlist != NULL) {
		qnr = levlist->qnumber;
		qebene = levlist->lev;
		qliste = levlist->qlist;
		while (qliste != NULL) {
			q = qliste->q;
			qpos = qliste->pos;
			qliste->xkoor = round(DIST + qebene * lendist);
			qliste->ykoor = round(MaxWidG + DIST - qpos * widdist);
			qliste = qliste->vertsucc;
		}
		levlist = levlist->levsucc;
	}
} /* GraphKoord */



static void GraphSymmKoord(plevel levlist)
{
	int			width, length, qnr, qebene, qpos, q, MaxLenG, MaxWidG;
	double		widdist, lendist;
	plevel		levhelp;
	pvertex		qliste;

	MaxLenG = XGRMAX - 2 * DIST;
	MaxWidG = YGRMAX - 2 * DIST;
	width = length = 0;
	levhelp = levlist;
	while (levhelp != NULL) {
		length++;
		if (levhelp->qnumber > width)
			width = levhelp->qnumber;
		levhelp = levhelp->levsucc;
	}
	lendist = (double)MaxLenG / (double)(length + 1);
	widdist = (double)MaxWidG / (double)( 2 * (width + 1));
	while (levlist != NULL) {
		qnr = levlist->qnumber;
		qebene = levlist->lev;
		qliste = levlist->qlist;
		while (qliste != NULL) {
			q = qliste->q;
			qpos = qliste->pos;
			qliste->xkoor = round(DIST + qebene * lendist);
			qliste->ykoor = round(MaxWidG + DIST - (qpos + 1) * widdist);
			qliste = qliste->vertsucc;
		}
		levlist = levlist->levsucc;
	}
} /* GraphSymmKoord */



static void FindLoopAngle(ptupel a, plevel levlist)
{
	int					i, j = 0, count, state; /* Zur Vorsicht initialisiert */
	double				anglehelp, angle ;
	strg				levelst;
	plist				prelist, levllist, suclist, looplist;
	pvertex				qlist, sucvert, prevert;
	matrixrealarray		anglearray;

	levelst[0] = 'l';
	while (levlist != NULL) {
		qlist = levlist->qlist;
		while (qlist != NULL) {
			if (qlist->barryleft == 1) {
				count = 1;
				state = qlist->q;
				if (state == a.qa) {
					anglearray[count] = 180;
					count++;
				}
				suclist = qlist->suc;
				while (suclist != NULL) {
					sucvert = suclist->edge;
					FindAngle(qlist->xkoor, qlist->ykoor, sucvert->xkoor, sucvert->ykoor, &angle);
					anglearray[count] = angle;
					count++;
					suclist = suclist->listsucc;
				}
				levllist = qlist->levl;
				while (levllist != NULL) {
					angle = (levllist->edgeint > 0) ? 180 : 0;
					anglearray[count] = angle;
					count++;
					levllist = levllist->listsucc;
				}
				prelist = qlist->pre;
				while (prelist != NULL) {
					prevert = prelist->edge;
					if (prevert->q != state) {
						if (prelist->edgest[0] == levelst[0])
							angle = (prevert->pos > qlist->pos) ? 180 :  0;
						else
							FindAngle(qlist->xkoor, qlist->ykoor, prevert->xkoor, prevert->ykoor, &angle);
						anglearray[count] = angle;
						count++;
						PListElemDel(&qlist->pre, prevert);
					}
					prelist = prelist->listsucc;
				}
				count--;
				for (i=1; i <= count; i++)			/* Sortieren der Winkel eines Knotens */
					for (j=1; j <= count - i; j++)
						if (anglearray[j+1] < anglearray[j]) {
							anglehelp = anglearray[j + 1];
							anglearray[j + 1] = anglearray[j];
							anglearray[j] = anglehelp;
						}
				angle = 0;
				for (i=1; i <= count - 1; i++) {
					anglehelp = anglearray[i + 1] - anglearray[i];
					if (anglehelp > angle) {
						angle = anglehelp;
						j = i;
					}
				}
				anglehelp = anglearray[1] + 360 - anglearray[count];
				if (anglehelp > angle) {
					angle = anglehelp;
					j = count;
				}
				looplist = qlist->pre;
				looplist->edgeint = round(anglearray[j] + (angle / 2.0));
				if (looplist->edgeint > 360)
					looplist->edgeint = looplist->edgeint - 360;
			}
			qlist = qlist->vertsucc;
		}
		levlist = levlist->levsucc;
	}
} /* FindLoopAngle */



void BuildAutomata(ptupel a, plevel levlist, matrixlistarray matrix, boolean symm)
	/* matrix ist var-Parameter */
{
	DrLevListPosSort(levlist);
	AddTransition(a, levlist, matrix);
	FindLevelEdgeRadius(levlist);
	if (!symm)
		GraphKoord(levlist);
	else
		GraphSymmKoord(levlist);
	FindLoopAngle(a, levlist);
} /* BuildAutomata */
