/******************************************************************************/
/*  %W% %D% %T%  */
/*                                                                            */
/*                                                                            */
/*                                                                            */
/*  XWindows Programm:   Raphael M. Grochtmann     Aachen, 31.08.1990         */
/*                                                                            */
/*                                                                            */
/*                                                                            */
/*                                   XAMD                                     */
/*                                                                            */
/*                            ( AMORE_Display )                               */
/*                    Schnittstelle zw. DFA und XWindows                      */
/*                                                                            */
/*             ( DFA - Display_Finite_Automata von Vera Kell )                */
/*                                                                            */
/*                                                                            */
/*                                                                            */
/******************************************************************************/

/* Schreibweisen:
		"Dir_Sym", "Dir_Nosym", "Nodir_Sym", "Nodir_Nosym"	Sind Namen der jeweiligen Widgets,
		dir_sym_w, dir_nosym_w, nodir_sym, nodir_nosym_w	sind die Widgets selbst;
		DIRSYM, DIRNOSYM, NODIRSYM, NODIRNOSYM				sind Konstanten aus ext.h zur
															Bestimmung des Darstellungsmodus.
*/


#include <stdio.h>
#ifdef SUN4				/* SunOS 4.x, hlg, 17. Aug. 94 */
#include <sys/file.h>			/* fuer access und R_OK */
#endif

#ifdef SUN5                            /* SunOS 5.x, hlg, 17. Aug. 94 */
#include <unistd.h>                     /* fuer access und R_OK  Lu-Go */
#endif

#include <X11/Intrinsic.h>      /* Intrinsics Definitions */
#include <X11/StringDefs.h>     /* Standard Name-String definitions */

#include <X11/Xaw/Form.h>		/* Athena Widgets ... */
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/StripChart.h>

#include "ext.h"				/* fuer die defines: DIRSYM, DIRNOSYM,... & PRINT, DONTPRINT */
#include "dfa.h"				/* InitDFA(), DoDFA(), full_filename, get_full_filename() */

/* globale Variablen, fuer die es KEINE Deklaration in ext.h gibt! */
GC			gc;						/* wird nur noch in xgraphics.c verwendet */
Display		*display;				/* wird nur noch in xgraphics.c verwendet */
Window		drawable;				/* wird nur noch in xgraphics.c verwendet */



/* globale Variablen (Deklaration in main.h) */
int			XGRMAX, YGRMAX; /* dfa.c, draw.c */
double		ARROWLEN, ARREDLEN, STATELEN, STSTOFFS, CURVCONST; /* dfa.c */
double		DIST; /* draw.c */
double		TEXTWIDTH, TEXTHEIGHT, FTBGOFFS, LOOPRADOFFS, DARROFFS; /* xgraph.c */
double		SKAL_FAKTOR; /* xgraph.c */



/* lokale Variablen */
static Widget		dir_sym_w, canvas_w;
static int			mode = 0;		/* Initialisierung mit 0, */
									/* damit mind. einmal draw_proc ausgefuehrt wird */
static Dimension	canvas_width, canvas_height;



static void draw_proc(w,  client_data, call_data)
	Widget		w;
	caddr_t		client_data, call_data;
{
	int		new_mode;
	Arg		args[2];

	new_mode = (int)XawToggleGetCurrent(dir_sym_w);
	if (new_mode != 0) {
			mode = new_mode;
			XtSetArg(args[0], XtNwidth, &canvas_width);		/* momentane Groesse ermitteln */
			XtSetArg(args[1], XtNheight, &canvas_height);	/* und Pseudokonst. neu skalieren */
			XtGetValues(canvas_w, args, 2);
			XGRMAX = (int)canvas_width;
			YGRMAX = (int)canvas_height;
			SKAL_FAKTOR = ((double)canvas_width / 1000.0 + (double)canvas_height / 700.0) / 2.0;
			ARROWLEN	=  8 * SKAL_FAKTOR;
			ARREDLEN	= 18 * SKAL_FAKTOR;
			STATELEN	= 12 * SKAL_FAKTOR;
			STSTOFFS	=  3 * SKAL_FAKTOR;
			CURVCONST	= 20 * SKAL_FAKTOR;
			DIST		=  4 * SKAL_FAKTOR;
			TEXTWIDTH	=  6;
			TEXTHEIGHT	=  8;
			FTBGOFFS	=  2 * SKAL_FAKTOR;
			LOOPRADOFFS	=  2 * SKAL_FAKTOR;
			DARROFFS	=  3 * SKAL_FAKTOR;
			XClearWindow(display, drawable);
			DoDFA(mode, DONTPRINT);
	}
} /*draw_proc*/

static void print_proc(w,  client_data, call_data)
	Widget		w;
	caddr_t		client_data, call_data;
{
	DoDFA(mode, PRINT);
} /*print_proc*/



static void quit_proc(w,  client_data, call_data)
	Widget		w;
	caddr_t		client_data, call_data;
{
	exit(0);
} /*quit_proc*/



static void expose(w, client_data, event)
Widget w;
caddr_t client_data;
XEvent *event;

{

        if (event->type == Expose)
                draw_proc(w, client_data);
} /*expose*/


main(argc, argv)
	int		argc;
	char	**argv;
{
	Widget		topLevel_w, form_w;
	Widget		print_w, quit_w;
	Widget		label_w, version_w;
	Widget		dir_nosym_w, nodir_sym_w, nodir_nosym_w;
	Arg			args[2];
	XGCValues	gcv;

	if (argc == 1) {
		fprintf(stderr, "*** ERROR: argument automata_file_name is missing ***\n");
		exit(1);
	} else {
		get_full_filename(argv[1]);
		if (access(full_filename, R_OK)) {
			fprintf(stderr, "*** ERROR: could not read - access %s *** \n", full_filename);
			exit(1);
		};
	};
	InitDFA();


	topLevel_w = XtInitialize(argv[0], "XAmd", NULL, 0, &argc, argv);

	form_w = XtCreateManagedWidget("form", formWidgetClass, topLevel_w, NULL, 0);

	XtSetArg(args[0], XtNradioData, DIRSYM);
	dir_sym_w		= XtCreateManagedWidget("Dir_Sym", toggleWidgetClass, form_w, args, 1);
	XtSetArg(args[0], XtNradioData, DIRNOSYM);
	dir_nosym_w		= XtCreateManagedWidget("Dir_Nosym", toggleWidgetClass, form_w, args, 1);
	XtSetArg(args[0], XtNradioData, NODIRSYM);
	nodir_sym_w		= XtCreateManagedWidget("Nodir_Sym", toggleWidgetClass, form_w, args, 1);
	XtSetArg(args[0], XtNradioData, NODIRNOSYM);
	nodir_nosym_w	= XtCreateManagedWidget("Nodir_Nosym", toggleWidgetClass, form_w, args, 1);

	canvas_w		= XtCreateManagedWidget("Canvas", stripChartWidgetClass, form_w, NULL, 0);
	XtSetArg(args[0], XtNlabel, argv[1]);	/* Name des Automaten */
	label_w			= XtCreateManagedWidget("Name", labelWidgetClass, form_w, args, 1);
	print_w			= XtCreateManagedWidget("Print", commandWidgetClass, form_w, NULL, 0);
	quit_w			= XtCreateManagedWidget("Quit", commandWidgetClass, form_w, NULL, 0);
	version_w               = XtCreateManagedWidget("Version", labelWidgetClass, form_w, NULL, 0);

	XtAddCallback(dir_sym_w, XtNcallback, draw_proc, NULL);
	XtAddCallback(dir_nosym_w, XtNcallback, draw_proc, NULL);
	XtAddCallback(nodir_sym_w, XtNcallback, draw_proc, NULL);
	XtAddCallback(nodir_nosym_w, XtNcallback, draw_proc, NULL);
	XtAddCallback(print_w, XtNcallback, print_proc, NULL);
	XtAddCallback(quit_w, XtNcallback, quit_proc, NULL);
 	XtAddEventHandler(canvas_w, ExposureMask, FALSE, expose, NULL);

 	XtRealizeWidget(topLevel_w);

	display = XtDisplay(canvas_w);
	drawable  = XtWindow(canvas_w);
	gcv.foreground = 1;		/* sonst weiss auf weiss ! */
	gc = XCreateGC(display, DefaultRootWindow(display), GCForeground, &gcv);

	XtMainLoop();
}
