/* cons.h 3.1 (KIEL) 01.95 */
/******************************************************************/
/* basic constants:
 * ALPHSIZE: size of alphabet
 * NABBR: number of abbriviations in the definition of reg. expressions
 * MAXLET: the character to NABBR (please edit positions XXX MAXLET)
 * NLANG: number of languages
 * NLEN:  max. number of charaters in a language name
 * REFLEN:  max. number of charaters in reference
 * PATHLEN: max. chars in (full) pathname
 * DEFAULT_PLINES: lines on the default printer device
 * DEFAULT_PCOLS: columns on the default printer device
 * NULL:  nil value for pointer
 * TABLESWITCH: if monoid > TABLESWITCH the multtable will not be shown full 
 * LINELEN: the length of lines in amr-files
 */
#define ALPHSIZE	26
#define NABBR	12
#define MAXLET  'L'  /* search for XXX MAXLET */
#define NLANG	9
#define NLEN	8
#define REFLEN 60
#define PATHLEN	255
#define DEFAULT_PLINES	66
#define DEFAULT_PCOLS	80
#ifdef DOS			    /* Lu-Go */
#define LINES 24		    /* Lu-Go */
#define COLS 80 		    /* Lu-Go */
#endif				    /* Lu-Go */

#ifndef NULL
#define NULL	0
#endif
#define TABLESWITCH 100

#define LINELEN 100
#define LINELEN1 (LINELEN+2)

/* system constants:
 * SIZEOFBYTE: #bits in char
 * PILEN: max. length of a posint or int  number
 * PIF: print and scan format for posint in format-strings
 * PIFMT,IFMT: print and scan format for posint and int (inclusive a newline)
 * BACKSPACE: keyboard code for <BACKSPACE> 
 * PATHSTR: separator for pathnames
 */
#define SIZEOFBYTE	8
#define PILEN		7
#define PIF		"%u"
#define PIFMT		"%u\n"
#define IFMT 		"%d\n"

#ifdef DOS				    /* Lu-Go */
#define BACKSPACE	'\b'		    /* Lu-Go */
#define PATHSTR 	"\\"		    /* Lu-Go */
#endif					    /* Lu-Go */

#ifdef UNIX				    /* Lu-Go */
#define BACKSPACE	(010)               /* LU-Go */
#define PATHSTR 	"/"                 /* Lu-Go */
#endif                                      /* Lu-Go */


/* the boolean values
 * TRUE, FALSE normal boolean values
 * UN_KNOWN is a boolx value with meaning: not yet computed
 * ISQUIT  is a boolx value with meaning: stop the computation
 * *MASK: masks for final, initial, etc. states
 */
#ifndef TRUE
# define FALSE	(0)
# define TRUE	(1)
#endif

#define UN_KNOWN (2)
#define ISQUIT	 (2)

#define FINMASK	0x1
#define INMASK	0x2

/* the symbols for regular expressions
 *
 * spacech : space
 * unionch : union
 * concatch: concatenation
 * starch  : Kleene-star
 * plusch  : Kleene-plus (L<plusch> == L<starch> <minusch> <epsch>
 * complch : complement
 * insecch : intersection
 * minusch : set difference
 * langch  : the empty set
 * epsch   : the empty word
 */
#define spacech		' '
#define unionch		'U'
#define concatch	'.'
#define starch		'*'
#define plusch		'+'
#define complch		'~'
#define insecch		'&'
#define minusch		'-'
#define langch		'%'
#define epsch		'@'
#define endch           '$'
#define leftbrch        '('
#define rightbrch       ')'
#define identch         '?'
/* Entries in the precedence table of the parser */
#define  EQUAL          0 
#define  LESS           1
#define  GREATER        2
#define  UNUSED         3
#define  UNBALBR        4       /* Unbalanced left bracket */
#define  UNBARBR        5       /* Unbalanced right bracket */
#define  ENDTEXT        6

/* stringform
 * strcomp: complement 
 * strunion: union
 * strspaunion: <space>union<space>
 * streps: empty word
 * strspaeps: <space> empty word
 * straddeps: <epsch><unionch>
 * strcataddeps: <concatch><epsch><unionch>
 * stralph: A<starch>
 * stropalph: (A<starch>
 * stralphcl: A<starch>)
 * strinter: intersection
 * strminus: set difference
 * strstar: star
 * strplus: plus
 * strconcat: concatenation char
 * emptysetstr: string for the empty set
 */

#define strcomp		"~"
#define strunion	"U"
#define strspaunion	" U "
#define streps		"@"
#define strspaeps	" @"
#define straddeps 	"@U"
#define strcataddeps	".@U"
#define stralph		"A*"
#define stropalph	"(A*"
#define stralphcl   	"A*)"
#define strinter	" & "
#define strminus	" - "
#define strstar		"*"
#define strplus		"+"
#define strconcat	"."
#define emptysetstr	"%"

/* mnemonics:
 * *INP : see types.h struct regset
 * *MENU: the menus
 * *WIN : window names
 * (LASTROW,LRCOL) is the screenposition where instructions,
 * little errormessages, etc. in hole screen applications appear
 * (not in a menu, but e.g. in edit)
 */
#define REXINP		0
#define DFAINP		1
#define NFAINP		2
#define OREXINP		10
#define ODFAINP		11
#define ONFAINP		12
#define OENFAINP	13

#define TESTMENU	'0'
#define LANGMENU	'1'
#define AUTMENU 	'2'
#define MAUTMENU	'3'
#define REXMENU 	'4'
#define MREXMENU 	'5'
#define ASSIGNMENU	'6'
#define VERSMENU        '7'

#define OWIN		'O'
#define TWIN		'T'
#define LWIN		'L'
#define LANG		'C'
#define EWIN		'E'
#define AWIN		'A'
#define MWIN		'M'
#define UWIN		'U'
#define BWIN		'B'

#define LASTROW		LINES-3
#define LRCOL  		3

/* special positons */
#define TITLEROW	1
#define TITLECOL	10
#define HELPROW		22              
#define HELPCOL		2   
#define INPROW		23              
#define INPCOL		2
#define INPOFF		10

/* window: languages
 * IDX* is used to write the names of the languages
 * MARKCOL: column for the mark of the  current language
 */
#define IDXROW  5
#define IDXCOL  67
#define MARKCOL 60


/*************
 ************* AUTMENU & REXMENU
 *************/

/* startposition for questions (row,col)
 * startposition for answer (icol)
 * MOD: modified language info
 * NAM: name of language
 * REF: reference
 * ALP: size of alphabet
 * QNO: number of automaton states
 * CON: confirm data
 */
#define MODROW		3
#define MODCOL		5

#define NAMROW		8
#define NAMCOL		5
#define INAMCOL		27

#define REFROW		9
#define REFCOL		5
#define IREFCOL		16

#define ALPROW		11
#define ALPCOL		5
#define IALPCOL		27

#define QNOROW		15	
#define QNOCOL		5
#define IQNOCOL		33

#define CONROW		20
#define CONCOL		20
#define ICONCOL		51


/*************
 ************* EDITORS
 *************/
/* qch,qtxt : quit-character and the name of it
 */
#define qch		'\033'
#define qtxt		"<ESC>"

/*************
 ************* EDITOR DEFAULTS
 *************/
/* the constants:
 * STDROWJUMP: distance between rows
 * STD?COLJUMP: distance between cols (?==D:dfa; ?==N:nfa)
 * STARTCOL : in this column the statenames start
 *            in this column the wordnames start (monoid)
 * INFOROW  : this row contains information about the automaton
 * STARTROW : this row contains the alphabet elements
              this row contatins statenames (monoid)
 * COLSUB   : number of not-useable colums (>STARTCOL)
 * ROWSUB   : number of not-useable rows  (>=STARTROW+#LASTROW's)
 * MAXNDSPLIT: number of states shown in ndet transitions
 */
#define STDROWJUMP	2
#define STDDCOLJUMP	9
#define STDNCOLJUMP	15
#define STARTCOL	5
#define INFOROW		3
#define STARTROW	5
#define COLSUB		6
#define ROWSUB		8
#define MAXNDSPLIT	3

/*************
 ************* AMR TEXT
 *************/
/* starting points for the array text */ /* old values reestablished by oma */
#define REGNO	0
#define DFANO	6
#define NFANO	14
#define DCLNO	22
#define MONNO	31
#define SFENO   46
#define GLBNO	56
 /* CUR_VERSION must be the index of the array versiontext where the
 headline of the current version of amore can be found */

#define CUR_VERSION 2            /* --- Bordeaux : 1995 --- */    
#define LEN_OF_HEADER 24



