/* edsfx.c 3.0 (KIEL) 11.94 */

#include "ext.h"

/* functions included:
 *      static void prepareconcat()
 *      static void spacecopy()
 *	static void concat()
 *	static void letterappend()
 *      static void epscompute()
 *      static void gen2expcompute()
 *	static int  no2repcompute()
 *	static void no2expadd()
 *	static void explengthcompute()
 *	static void printtitle()
 *	       void editsfx()
 */

/* 
 * gen2expcompute : compute for each generator an expression
 *                  which consists of those letters with the same
 *                  transformation
 *
 * no2repcompute  : compute for an element with representative w the string L(w)
 * 
 * no2expadd      : transforms the represantation of a starfree expression
 *    	           in a string and show/print it 
 *
 * explengthcompute : computes the length of the expressions u,v,w 
 *
 * outsfree : main procedure
 *
 *   format and static output constants 
 *                    
 *  L = ....  title
 *      right side starts at columne firsttitle
 *      left side starts at columne colfirst
 *
 *  L(w) = .....
 *      right side starts at columne firstcol
 *      left side starts at columne colfirst
 *      equality sign at position equalcol
 *
 *      if words are long output in the form  L( .. ) 
 *                                               = ...
 *       else             output in the form  L( .. ) = ...
 *
 */ 

/******************************************************************/
 static monoid stamon;
 static dfa stadfa;
 static starfexp stastar;

/* static variables used for the computation of the expressiones */
 static array no2ulength,no2vlength,no2wlength;
 static arrayofstring gen2letterexp,no2rep; 
 static array genlength,replength; /* length of gen2letterexp, no2rep */
 static string epsexp; /* contains L(1) */
 static boole emptyeps; /* TRUE iff L(1) = @ */
 static b_array comp; /* comp[n] = TRUE <=> starfree expr. for element n is computed */

/* static flags for the output */
 static boole expan; /* starfree expression is shown expanded */
 static boole printer; /* printer or screen */
 static boole repeattitle; /* repeat output: L = .. */
 static boole newl;     /* newline after "L(   ) =  " */
 static boole inprinttitle; /* set if in procedure printtitle */

/* static variables for the output */
 static int rowco; /* rowcounter */
 static int colco; /* colcounter */ 

/* static constants for the output */
 static int colfirst; /* first columne in use */
 static int collast;  /* last columne in use */
 static int rowfirst; /* first row in use */
 static int rowlast;  /* last row in use */
 static int equalcol; /* columne for the equality sign */
 static int firstcol; /* first columne for the output of an expresssion */
 static int firsttitle; /* first columne for the output of the title */

#define with expan&&(n||(!emptyeps))


/******************************************************************/
/******************************************************************/
 static posint spacelen; /* first free place in space */
 static string space;

static void prepareconcat()
 /* compute size of space = 
  * maximum of
  *   - length of (a1+a2+..+an) < stamon->sno*(letterlength+1)
  *   - length of maximal representative*letterlength
  */
{
 posint maxlen;
 posint letterlength;
 letterlength=(stamon->sno<=ALPHSIZE)? 1 : (strlen(pi2a(stamon->sno))+1);
 maxlen=stamon->no2length[stamon->mno-1]*letterlength; /* abbreviation */
 if(maxlen<(stamon->sno*(letterlength+1))) maxlen=stamon->sno*(letterlength+1);
 space=newstring(maxlen+10);
 spacelen=0;
}
/******************************************************************/
static void concat(s)
/* append s at position spacelen in space
 * spacelen is always the length of the string in space 
 */
string s;
{
 posint j=0;
 while(s[j]!='\0')space[spacelen++]=s[j++];
}

/******************************************************************/
static void spacecopy(s)
string* s; /* copy space to string s */
{
 posint j; 
 *s=newstring(spacelen+1);
 for(j=0;j<spacelen;j++)
   (*s)[j]=space[j];
 (*s)[spacelen]='\0';
 spacelen=0;
} 

/******************************************************************/
static void letterappend(letter)
posint letter;  /* add a letter to space */
{
 if(stamon->sno>ALPHSIZE)
   {
    space[spacelen++]='a';
    concat(pi2a(letter));
    }
 else
    space[spacelen++]=itoc[letter];
}

/******************************************************************/
static void epscompute()
{
 /* compute no2rep[0], epsexp
  *       for all letters whose transformation is the identity
  *
  *   if L(1) != @ 
  *       L(1)=~A*(a1+a2+a3+  +an)A* for all letters with transformation != identity
  *   if L(1) == @  then no2rep[0] = "" epsexp = ""
  */
 posint letter,run;
 posint count=0; /* count letters with transformation != identity */
 no2vlength[0]=0;
 no2wlength[0]=0;
 for(letter=1;letter<=stamon->sno;letter++) /* for all letters */
   if(stamon->let2gen[letter]) count++;
 if(count==stamon->sno) /* L(1) = @ */
   {
    /* compute no2rep */
    emptyeps = TRUE;
    no2rep[0]=newstring(1);
    no2rep[0]="";
    replength[0]=0;
    /* compute epsexp */
    no2ulength[0]=0;
    epsexp=newstring(1);
    epsexp="";
   }
 else
   {
    /* compute no2rep */
    no2rep[0]=newstring(5);
    no2rep[0]="L(1)";
    replength[0]=4;
    /* compute epsexp */
    if(!count) /* L(1) = A* */
      {
       emptyeps = FALSE;
       concat(stralph);
      }
    else
      {
       emptyeps = FALSE;
       concat(strcomp);
       concat(stropalph);  /* (A* */
       if(count>1) concat("(");
       run=0;
       for(letter=1;letter<=stamon->sno;letter++)
           if(stamon->let2gen[letter]!=0)
             { run++;
               letterappend(letter);
               if(run<count)concat(strspaunion);
             }
       if(count>1) concat(")");
       concat(stralphcl); /* A*) */
      }
    no2ulength[0]=spacelen;
    spacecopy(&epsexp);
   } 
}

/******************************************************************/
static void gen2expcompute()
/*  gen2letterexp[i]=(a1+a2+..+an) n>1    generator i has the same
 *               or = a1           n=1    transformation as the letters
 *                                         a1 .. an
 */
{
 posint gen;
 posint letter;
 posint count,count2;
 for(gen=1;gen<=stamon->gno;gen++) /* for all generators */
   {
    count=0;
    for(letter=1;letter<=stamon->sno;letter++) /* for all letters */
         if(stamon->let2gen[letter]==gen) count++;
    if(count>1) concat("(");
    count2=0;
    for(letter=1;letter<=stamon->sno;letter++)
      if(stamon->let2gen[letter]==gen)
         {
	  count2++;
          letterappend(letter);
          if(count2<count)concat(strunion);
	 }
    if(count>1)concat(")");
    genlength[letter]=spacelen;
    spacecopy(&gen2letterexp[gen]);
   } 
}

/******************************************************************/
static int no2repcompute()
/*  no2rep[n]=L(w) and w is a shortest represantative of melement n
 *  no2repcompute translates stamon->no2word[n] in a string
 *  return: maximal length of a element L(w) on the right side of an expression 
 */
{
 posint n;
 int max;
 max=(stadfa->final[stamon->no2trans[0][stadfa->init]])? 4 : 0;
 for(n=1;n<stamon->mno;n++)
   if(stastar->computed[n])
     { 
       concat("L(");
       concat(prword(n,stamon,FALSE)); 
       concat(")");
       if(stadfa->final[stamon->no2trans[n][stadfa->init]])
	  if(max<spacelen) max=spacelen;
       replength[n]=spacelen;
       spacecopy(&no2rep[n]);
     }
 return(max);  
}

/******************************************************************/
static void addtoken();

static void printtitle()
{
 boole mark=FALSE; /* + must be inserted */
 posint elem;
 rowco=rowfirst;
 colco=colfirst;
 inprinttitle=TRUE;
 if(!repeattitle)return; 
 addtoken(" L = " , firstcol, FALSE);
 for(elem=0;elem<stamon->mno;elem++)
    if(stadfa->final[stamon->no2trans[elem][stadfa->init]])
         {
          if(mark)  addtoken(strspaunion,firsttitle,FALSE);
	  else mark=TRUE;
          if(elem) addtoken(no2rep[elem],firsttitle,FALSE);
          else addtoken("L(1)",firsttitle,FALSE);
         }
/* test repeattitle */
 rowco+=2;
}

/******************************************************************/
static void newpage(firstpage)
boole firstpage;
{
 if(printer)
   {
    if(!firstpage)
       {
   
        pgbreak();
        gohorizontal();
        rowco=rowfirst;
        }
    }
 else
   {
    if(!firstpage)
      {
       move(LASTROW,1);
       refresh();
       (void)getch();
      }
    clear();
    maketitle(TRUE,"S T A R F R E E   E X P R E S S I O N");
    mvaddstr(LASTROW,5,"press any key to continue");
    rowco=rowfirst;
    if(inprinttitle) repeattitle=FALSE;
    else
       if(repeattitle) printtitle();
   }
}

/******************************************************************/
static void addtoken(s,first,devide)
/* add string s at position (rowco,colco)
 * if line to short for the token and not devide: 
 *             start at new line at position first
 */
string s; 
int first; /* first col if new line */
boole devide; /* if line to short advance one line */ 
{
 int i;
 int help=strlen(s);
 if(help>(collast-colco+1))
    if((!devide)&&(colco>first))
      {rowco++;colco=first;}
 for(i=0;i<help;i++)
   {
    if(colco==collast) 
     {colco=first;rowco++;}
    if(rowco==rowlast) 
      { newpage(FALSE); colco=first; }
    if(printer)
         pgmvaddch(rowco,colco++,s[i]);
    else
         mvaddch(rowco,colco++,s[i]);
   }
}
       
/******************************************************************/
static void explengthcompute(n)
posint n;
{
 posint ulength,vlength,wlength;
 posint ul=0;
 posint vl=0;
 posint wl=0;
 posint count=0;
 posint help;
 posint i;
 comp[n]=TRUE;
 /* abbreviations */
 ulength=stastar->ulength[n];
 vlength=stastar->vlength[n];
 wlength=stastar->w1length[n]+stastar->w0length[n];

 /* compute length of string u */

 if(wlength)ul++; /* brackets around u+v */
 if(ulength>1) ul+=2; /* brackets around u */
 ul += ((ulength-1)*3); /* ulength -1 strspaunion */
 for(i=0;i<ulength;i++)
   {
    help=stastar->u[n][count++];
    if(expan)
     {
      if(!comp[help])explengthcompute(help);
      ul += (2+no2ulength[help]+no2vlength[help]+no2wlength[help]);
     }
    else
      ul += replength[help];
    ul += genlength[stastar->u[n][count++]];
   }
 ul += 2; /* A* at the end of u  */

 /* compute length of string  v */

 vl += 5; /* " & A*" */
 if(vlength>1) vl+=2; /* brackets around v */
 vl += ((vlength-1)*3); /* vlength -1 strspaunion */
 count=0;
 for(i=0;i<vlength;i++)
   {
    vl += genlength[stastar->v[n][count++]];
    help=stastar->v[n][count++];
    if(expan)
     {
      if(!comp[help])explengthcompute(help);
      vl += (2+no2ulength[help]+no2vlength[help]+no2wlength[help]);
     }
    else
      vl += replength[help];
   }
 if(wlength)vl++; /* brackets around u+v */

 /* if(!wlength) then no2w[n]="" */

 if(wlength)
    {
     wl += 7; /* " - A* .. A* */
     if(wlength>1) wl += 2; /* brackets around w */
     wl += ((wlength-1)*3); /* wlength -1 strspaunion */
     count=0;
     for(i=0;i<stastar->w0length[n];i++)
        {
         wl += genlength[stastar->w0[n][count++]];
         help=stastar->w0[n][count++];
         if(expan)
          {
           if(!comp[help])explengthcompute(help);
           wl += (2+no2ulength[help]+no2vlength[help]+no2wlength[help]);
          }
         else
           wl += replength[help];
         wl += genlength[stastar->w0[n][count++]];
        }
     count=0;
     for(i=0;i<stastar->w1length[n];i++)
         wl += genlength[stastar->w1[n][count++]];
    }
 no2ulength[n]=ul;
 no2vlength[n]=vl;
 no2wlength[n]=wl;
 }

/******************************************************************/
static void no2expadd(n,out)
/* add the starfree expr for inversphi(n) 
 * 
 *  wlength   (uA*#A*v)-A*wA*
 * !wlength   uA*#A*v
 *  ulength>1  (u)  vlength,wlength>1 same way
 *  ulength=1   u     
 *
 *  see also expcompute(n) 
 *
 *  advance one line after u or v if necessary
 */ 
posint n;
boole out; /* out iff no2expadd is not called from no2expadd */
{
 posint ulength=stastar->ulength[n],vlength=stastar->vlength[n];
 posint wlength=stastar->w1length[n]+stastar->w0length[n];
 posint count=0;
 posint i;
 if(out) /* print left side */
  {
   colco = colfirst;
   if(n)
     addtoken(no2rep[n],colfirst,TRUE);
   else
     addtoken("L(1)",colfirst,TRUE);
   if(newl) rowco++;
   colco=equalcol;
   addtoken("=",equalcol,FALSE);
   colco=firstcol;
  }
 else
  if(with)addtoken("(",firstcol,TRUE); 

 if(n) /* not the idenitity */
   {
    /* compute string u */
   
    if(wlength) addtoken("(",firstcol,TRUE); 
    if(ulength>1) addtoken("(",firstcol,TRUE);
    count=0;
    for(i=0;i<ulength;i++)
      {
       if(i) addtoken(strspaunion,firstcol,TRUE);
       if(expan) no2expadd(stastar->u[n][count++],FALSE);
       else addtoken(no2rep[stastar->u[n][count++]],firstcol,TRUE);
       addtoken(gen2letterexp[stastar->u[n][count++]],firstcol,TRUE);
     }
    if(ulength>1) addtoken(")",firstcol,TRUE);
    addtoken(stralph,firstcol,TRUE);
   
    if(out)
      if(no2vlength[n]>(collast-colco+1)) /* v can`t be printed in the rest of the line */ 
        { rowco++; colco=firstcol; }
    
    /* compute string v */

    addtoken(strinter,firstcol,TRUE);
    addtoken(stralph,firstcol,TRUE);
    if(vlength>1) addtoken("(",firstcol,TRUE);
    count=0;
    for(i=0;i<vlength;i++)
      {
       if(i)addtoken(strspaunion,firstcol,TRUE);
       addtoken(gen2letterexp[stastar->v[n][count++]],firstcol,TRUE);
       if(expan) no2expadd(stastar->v[n][count++],FALSE);
       else addtoken(no2rep[stastar->v[n][count++]],firstcol,TRUE);
      }
    if(vlength>1) addtoken(")",firstcol,TRUE);
    if(wlength)addtoken(")",firstcol,TRUE); 
   
    if(out)
      if(no2wlength[n]>(collast-colco+1)) /* w can`t be printed in the rest of the line */ 
        { rowco++; colco=firstcol; }
   
      /* compute string w 
       * if(!wlength) then no2w[n]=""
       */
   
    if(wlength)
       { addtoken(strminus,firstcol,TRUE); addtoken(stralph,firstcol,TRUE);} 
    if(wlength>1) addtoken("(",firstcol,TRUE); /* brackets around w */
    count=0;
    for(i=0;i<stastar->w0length[n];i++)
       {
        if(i)addtoken(strspaunion,firstcol,TRUE);
        addtoken(gen2letterexp[stastar->w0[n][count++]],firstcol,TRUE);
        if(expan) no2expadd(stastar->w0[n][count++],FALSE);
        else addtoken(no2rep[stastar->w0[n][count++]],firstcol,TRUE);
        addtoken(gen2letterexp[stastar->w0[n][count++]],firstcol,TRUE);
       }
    count=0;
    if(stastar->w0length[n]&&stastar->w1length[n])
        addtoken(strspaunion,firstcol,TRUE);
    for(i=0;i<stastar->w1length[n];i++)
       {
        if(i)addtoken(strspaunion,firstcol,TRUE);
        addtoken(gen2letterexp[stastar->w1[n][count++]],firstcol,TRUE);
       }
    if(wlength>1)addtoken(")",firstcol,TRUE);
    if(wlength) addtoken(stralph,firstcol,TRUE); 
   }
 else /* identity */
   if(out)
     if(emptyeps)
        addtoken("@",firstcol,TRUE); 
     else 
	addtoken(strspaeps,firstcol,TRUE);
   else
     if(expan)
       addtoken(epsexp,firstcol,TRUE); 
     else
       addtoken(no2rep[0],firstcol,TRUE);
 if(out) rowco++;
 else  
     if(with)addtoken(")",firstcol,TRUE); 
}  /* no2expadd */

/******************************************************************/
void editsfx(indfa,mon,sfx,ex,pr)
dfa indfa;
monoid mon;
starfexp sfx;
boole ex; /* expand or not expand */
boole pr; /* printer or screen */
{
 posint elem;
 int help;
 /* set static variables */
 expan = ex;
 printer = pr;
 stadfa=indfa;
 stamon=mon;
 stastar=sfx;
 /* get memory */
 comp=newb_array(stamon->mno); 
 no2rep=newarrayofstring(stamon->mno);
 replength=newarray(stamon->mno);
 gen2letterexp=newarrayofstring(stamon->gno+1);
 genlength=newarray(stamon->gno+1);
 no2ulength=newarray(stamon->mno);
 no2vlength=newarray(stamon->mno);
 no2wlength=newarray(stamon->mno);
 prepareconcat();
 /* compute expressions */
 epscompute(); 
 gen2expcompute();
 help=no2repcompute();
 /* compute length of expressions */
 for(elem=1;elem<stamon->mno;elem++)
   if(stastar->computed[elem])
       if(!(comp[elem]))explengthcompute(elem);
 /* set output constants  */
 rowfirst = (printer)? 1 : 4; 
 rowlast =  (printer)? plines-3 : LASTROW;
 colfirst = 1; 
 collast = (printer)? pcols-3 : COLS -3; 
 firsttitle = colfirst + 5;
 equalcol = help + colfirst + 3;
 if((2*equalcol)>collast) 
   { 
    equalcol = 10; 
    newl = TRUE; 
   }
 else
   newl=FALSE;
 firstcol = equalcol + 2; 
 /* start output */ 
 if(!printer) clear();
 repeattitle=TRUE;
 inprinttitle=FALSE;
 newpage(TRUE);
 printtitle();
 for(elem=0;elem<stamon->mno;elem++) 
    if(stastar->computed[elem])
        no2expadd(elem,TRUE);
 /* complete last page */
 if(printer)
    pgbreak(); 
 else
   { 
    move(rowlast,collast); 
    refresh();
    (void)getch();
   }
}  /* editsfx */
