/* 
   Bordeaux : 1995
   */

#include "ext.h"
#include "extbis.h"

#include "fa_word.h"
#include "fa_utils.h"
#include <stdlib.h>
#include <string.h>
#include <setjmp.h>


/* make a recursive deep scan of an automaton to search a word recognized by this 
   automaton : 
   state  = state where we are in the scan of the automaton global_X_dfa 
   length = lenght of the path followed to reach state */
static void recursive_deep(posint state, int length);

/* global variables for recursive_deep and fa_word_find_a_word_in_dfa */
static posint  global_init;  /* initial state of the scaned automaton */
static posint  global_sno;   /* number of letters of the alphabet of this 
				automaton */
static posint  global_well;  /* well state of the automaton this automaton */
static ddelta  global_delta; /* transition function of this automaton */
static mrkfin  global_final; /* array of final states of this automaton */
static boole  *visited;      /* array of boolean to mark the visited states in the 
				recursive deep scan of this automaton */
static jmp_buf buff;         /* to stop the recursive scan of this automaton */
static fa_word word;         /* result of fa_word_find_a_word_in_dfa */


/* compute the maximum number of char in the representation of a letter, according
 to the size of the alphabet */
static int nb_max_char_in_a_letter(posint sno);

/* generic function for building special fa_word of one letter */
static fa_word generic_special_fa_word(int letter);


/* ---------------------------- Global functions ------------------------------- */


fa_word fa_word_find_a_word_in_dfa(dfa X_dfa, boole eps)
{
  posint qno; /* number of states of X_dfa */
  int i;

  /* X_dfa represents the empty language */
  if ( fa_utils_test_empty_dfa(X_dfa) == TRUE ) 
    return fa_word_empty_language();

  global_init  = X_dfa->init;
  global_final = X_dfa->final;
  
  /* if there is the empty-word in X_dfa */
  if (fa_utils_test_eps_dfa(X_dfa) == TRUE || 
      eps == TRUE && fa_utils_test_if_eps_in_dfa(X_dfa) == TRUE )
    return fa_word_eps();

  global_sno   = X_dfa->sno;
  global_delta = X_dfa->delta;
  global_well  = fa_utils_well(X_dfa);
  qno   = X_dfa->qno;

  /* the number of states of global_X_dfa is (qno + 1) */
  visited = (boole *) malloc((qno+1) * sizeof(boole)); 
  if( visited == NULL )
    free_resources(1);
      

  /* in a recursive deep scan of global_X_dfa, the number of steps of the scan
     will be inferior or equal to  qno + 1. Morever, we reserve the place for the
     end character (END_OF_FA_WORD) */
  word = (fa_word) malloc((qno + 2) * sizeof(int));
  if( word == NULL )
    {
      free(visited);
      free_resources(1);
    }

  /* initialisation of the array visited */
  for ( i = 0 ; i <= qno ; i++ )
    {
      visited[i] = FALSE;
      word[i] = END_OF_FA_WORD;
    }
  word[i] = END_OF_FA_WORD;

  if ( setjmp(buff) == 0 )
    recursive_deep(global_init, 0);
  else
    {
      free(visited);
      return word;
    }
}
	   

char *fa_word_convert_fa_word_to_string(fa_word src, posint sizeal)
{
  int   nb_char_letter;  /* max number number of char in a letter */
  int   i;
  int   length;    /* length = number of letter of the result word */
  int   cursor;    /* cursor to scan src and build the result */
  int   letter_id; /* letter (int) to convert in a char or char * */
  char *result;    /* result of the conversion of src */

  nb_char_letter = nb_max_char_in_a_letter(sizeal);

  length = fa_word_len(src);

  /* allocation of the maximal size necessary for result */
  result = (char *) malloc(((length * nb_char_letter) + 1) * sizeof(char));
  if( result == NULL )
    free_resources(1);      

  cursor = 0;
  letter_id = src[cursor];  
  if ( sizeal <= ALPHSIZE )
    /* if the alphabet is less than ALPSIZE, the letters are char, else (char *) */
    {      
      while( letter_id != END_OF_FA_WORD )
	{
	  switch( letter_id )
	    {
	    case PI_SEPARATOR1 :
	      result[cursor++] = SEPARATOR1;
	      break;
	    case PI_SEPARATOR2 :
	      result[cursor++] = SEPARATOR2;
	      break;
	    default :
	      result[cursor++] = itoc[letter_id];     
	    }
	      
	  letter_id = src[cursor];
	}
      result[cursor] = END_OF_STRING;
    }
  else
    {
      int cursor2 = 0;
      while( letter_id != END_OF_FA_WORD )
	{	  
	  switch( letter_id )
	    {
	    case PI_SEPARATOR1 :
	      result[cursor2++] = SEPARATOR1;
	      break;
	    case PI_SEPARATOR2 :
	      result[cursor2++] = SEPARATOR2;
	      break;
	    default :
	      {
		char *indice = pi2a(letter_id);
		strcat(result, "a");
		strcat(result, indice);
		cursor2 += 1 + strlen(indice);
		free(indice);
	      }
	    }

	  letter_id = src[++cursor];	  
	}
      result[cursor2] = END_OF_STRING;
    }

  /* we adjust the allocation for result with its real size */
  result = (char *) realloc(result, (strlen(result) + 1) * sizeof(char));
  if( result == NULL )
    free_resources(1);      

  return result;
}



fa_word fa_word_eps()
{
  return generic_special_fa_word(EPS_FA_WORD);
}


fa_word fa_word_empty_language()
{
  return generic_special_fa_word(EMPTY_LANGUAGE_FA_WORD);
}


boole fa_word_test_eps(fa_word word)
{
  return ( word[0] == EPS_FA_WORD && word[1] == END_OF_FA_WORD );
}


boole fa_word_test_empty_language(fa_word word)
{
  return ( word[0] == EMPTY_LANGUAGE_FA_WORD && word[1] == END_OF_FA_WORD );
}

int fa_word_len (fa_word word)
{
  int result;

  if ( fa_word_test_empty_language(word) == TRUE )
    result =  EMPTY_LANGUAGE_FA_WORD_LEN;
  else
    if ( fa_word_test_eps(word) == TRUE )
      result = EPS_FA_WORD_LEN;
    else
      {
	int l = 0;

	while( word[l++] != END_OF_FA_WORD);
	result = l-1;
      }

  return result;
}



fa_word fa_word_remove_letter(fa_word src, posint letter)
{
  int     cursor1; /* cursor to build result */
  int     cursor2; /* cursor to scan src */
  fa_word result;

  cursor1 = 0;
  cursor2 = 0;

  while( src[cursor2] != END_OF_FA_WORD )
    if( src[cursor2++] != letter )
      cursor1++;

  if( cursor1 != 0 )
    {
      result = (fa_word) malloc(cursor1 * sizeof(int));

      cursor1 = 0;
      cursor2 = 0;
      
      while( src[cursor2] != END_OF_FA_WORD )
	if( src[cursor2] != letter )
	  result[cursor1++] = src[cursor2++];
	else
	  cursor2++;     
      result[cursor1] = END_OF_FA_WORD;
    }
  else
    result = fa_word_eps();

  return result;
}



/* ---------------------------- Static functions ------------------------------- */


static void recursive_deep(posint state, int length)
{
  posint j;

  visited[state] = TRUE;

  for ( j = 1 ; j <= global_sno ; j++ )
    {
      posint new_state = global_delta[j][state];

      if( new_state != global_well )	/* we avoid the well state */
	if ( isfinal(global_final[new_state]) == TRUE )
	  {
	    word[length] = j;      
	    longjmp(buff, 1); /* the scan is stopped when a final state 
				 is encountered */ 
	  }
	else if( visited[new_state] == FALSE )
	  {
	    word[length] = j;      
	    recursive_deep(new_state, length + 1);
	  }	    
    }
}


static int nb_max_char_in_a_letter(posint sno)
{
  int result = 1;

  if( sno > ALPHSIZE )
    {
      char *tmp;
      int   result;

      tmp = pi2a(sno);
      result = strlen(tmp);
      free(tmp);
    }

  return result;
}


static fa_word generic_special_fa_word(int letter)
{
  fa_word result;

  result = (fa_word) malloc(2 * sizeof(int));
  if( result == NULL )
    free_resources(1);
  
  result[0] = letter;
  result[1] = END_OF_FA_WORD;

  return result;
}







