#ifndef FA_WORD_H
#define FA_WORD_H

/*
   Bordeaux : 1995
   */

#include "ext.h"
#include "extbis.h"

/* This module provides functions to work with the word extracted of a FA under 
   the form of a fa_word (this type is defined in extbis.h) */ 

/* extract a word of the dfa X_dfa under the form of a fa_word :
   if eps = TRUE, the empty word (EPS_FA_WORD) will be accepted as result
   if X_dfa is the empty-word DFA : EPS_FA_WORD is the result
   if X_dfa is the empty language, a special fa_word will be the result:
   EMPTY_LANGUAGE_FA_WORD
   else the result is the first word of X_dfa in the lexicographical order.
   Assume that X_dfa is minimal */
fa_word fa_word_find_a_word_in_dfa(dfa X_dfa, boole eps);

/* convert a variable of the type fa_word into a (char *) in the alphabet with
   size_al letters : none of the elements of src must be superior to size_al 
   src must be different of EPS_FA_WORD and EMPTY_FA_WORD */
char *fa_word_convert_fa_word_to_string(fa_word src, posint sizeal);

/* affect the empty word (EPS_FA_WORD) to a fa_word */
fa_word fa_word_eps(void);

/* affect the special fa_word corresponding to an empty language 
   (EMPTY_LANGUAGE_FA_WORD)*/
fa_word fa_word_empty_language(void);

/* test if a fa_word is the empty_word */
boole fa_word_test_eps(fa_word word);

/* test if a fa_word is the special fa_word corresponding to an empty language */
boole fa_word_test_empty_language(fa_word word);

/* compute the length (number of posint) of a variable of the type fa_word :
   if word is the epsilon word return EPS_FA_WORD_LEN
   if word is the empty_language word return EMPTY_LANGUAGE_FA_WORD_LEN */
int fa_word_len(fa_word word);

/* Little filter that remove a "letter" in a fa_word (src) */
fa_word fa_word_remove_letter(fa_word src, posint letter);


#endif
