/* mem.c 3.0 (KIEL) 11.94 */

#include "ext.h"
#include "extbis.h"

/* functions include:
 *
 * 	void	 freedfa()
 *	void	 freenfa()
 *	void	 freerex()
 *	void	 freesf()
 *	void	 freedcl()
 *	void	 freemon()
 *	language newlang()
 *	void 	 movelang()
 *	ddelta	 newddelta()
 *	ndelta	 newndelta()
 *	string	 newrexstr()
 *	mrkfin	 newfinal()
 *	dfa	 newdfa()
 *	nfa	 newnfa()
 *	regex	 newrex()
 *	monoid	 newmon()
 *	array	 newar()
 *	boole*	 newbarray()
 *	arrayofarray newarray1()
 *	d_class	 newdclass()
 *	dstruct	 newdstruct()
 *	d_list	 newdlist()
 *	starfexp newsfexp()
 *	darray	 newdarray()
 */

/* Bordeaux 1995 :
   initialize the field word_not_code of a new language */
static void init_word_not_code(language lan); 

static void init_word_not_code(language lan) 
{
  lan->word_not_code = (char *) malloc((strlen(WORD_NOT_CODE_DEFAULT) + 1) * sizeof(char));
  if( lan->word_not_code == NULL )
    free_resources(1);
  strcpy(lan->word_not_code, (char *) WORD_NOT_CODE_DEFAULT);
}


/******************************************************************/
/******************************************************************/
void
freedfa(da)
dfa da;
/* frees the memory used by the dfa da */
{ register posint i;
	dispose(da->final);
	for(i=1; i <= da->sno; ) dispose(da->delta[i++]);
        dispose(da->delta);
} /* freedfa */

/******************************************************************/
void
freenfa(na)
nfa na;
/* frees the memory used by the nfa na */
{ posint i,j;
	dispose(na->infin);
	for(j=(na->is_eps)?0:1; j <= na->sno;) {
		for(i=0; i <= na->qno; )
			dispose(na->delta[j][i++]);
		dispose(na->delta[j++]);
	}
	dispose(na->delta);
} /* freenfa */

/******************************************************************/
void
freerex(rx)
regex rx;
/* frees the memory used by the regular expression rx */
{ register int i;
	for(i=0; i<=rx->useda; i++) dispose(rx->abbr[i]);
	dispose(rx->rex);
	dispose(rx->exprex);
} /* freerex */

/******************************************************************/
void
freesf(sf,flag,mno)
starfexp sf;
boole flag;
posint mno;
/* frees the memory used by the starfree regular expression sf
 * flag == TRUE iff memory for the array's has been allocated
 * mno is the number of monoid elements
 */
{ register posint i;
	dispose(sf->computed);
	dispose(sf->ulength);
	dispose(sf->vlength);
	dispose(sf->w0length);
	dispose(sf->w1length);
	if (flag) for(i=0; i< mno; i++) {
		dispose(sf->u[i]);
		dispose(sf->v[i]);
		dispose(sf->w0[i]);
		dispose(sf->w1[i]);
	}
	dispose(sf->u);
	dispose(sf->v);
	dispose(sf->w0);
	dispose(sf->w1);
} /* freesf */

/******************************************************************/
void
freedcl(dcl,i)
dstruct dcl;
posint i;
/* frees the memory used by the D-class structure dcl
 * i is the number of the used entries in dclassarray
 */
{ register posint j;
	for(j=0; j < i ; j++) {
		dispose(dcl->dclassarray[j]->lrep);
		dispose(dcl->dclassarray[j]->rrep);
		dispose(dcl->dclassarray[j]->hclass);
	}
	for(j=0; j < dcl->dno; j++) dispose(dcl->dclassarray[j]);
	dispose(dcl->dclassarray);
	dispose(dcl);
} /* freedcl */

/******************************************************************/
void
freemon(mon)
monoid mon;
/* frees the memory used by the monoid mon
 */
{ register posint i;
	dispose(mon->generator);
	dispose(mon->let2gen);
	for(i=0; i< mon->mno;) {
	     dispose(mon->no2trans[i]);
	     dispose(mon->gensucc[i++]);
	}
	dispose(mon->gensucc);
	dispose(mon->no2trans);
	dispose(mon->lastletter);
	dispose(mon->no2length);
	dispose(mon->word);
	if(mon->relationcomputed) 
	  {dispose(mon->rs->rside);
	   dispose(mon->rs->lside);
           dispose(mon->rs);
          }
	if (mon->dclassiscomputed) 
		freedcl(mon->ds,mon->ds->dno);
} /* freemon */

/******************************************************************/
language
newlang(idx)
     int idx;
/* allocate memory for a new language */
{language hlp;
 posint i;
	
 hlp = langs[idx];

 for(i = 0; i < NLEN; hlp->name[i++] = '\0');
 for(i = 0; i < REFLEN; hlp->ref[i++] = '\0');

 hlp->saved  = FALSE;
 hlp->sizeal = 0;
 /* the properties of the language are UN_KNOWN */
 hlp->empty       = UN_KNOWN;
 hlp->full        = UN_KNOWN;
 hlp->folu        = UN_KNOWN;
 hlp->starfree    = UN_KNOWN;
 hlp->localtest   = UN_KNOWN;
 hlp->definite    = UN_KNOWN;
 hlp->revdefinite = UN_KNOWN;
 hlp->gendefinite = UN_KNOWN;
 hlp->dotdepth1   = UN_KNOWN;
 hlp->piecetest   = UN_KNOWN;
 hlp->nilpotent   = UN_KNOWN;
 /* --- Bordeaux : 1995 : start --- */
 hlp->code         = UN_KNOWN;  
 hlp->stack_code   = UN_KNOWN;  
 hlp->prefix_code  = UN_KNOWN;  
 hlp->suffix_code  = UN_KNOWN;  
 hlp->bifix_code   = UN_KNOWN;  
 hlp->p_infix_code = UN_KNOWN;  
 hlp->s_infix_code = UN_KNOWN;  
 hlp->infix_code   = UN_KNOWN;  
 hlp->outfix_code  = UN_KNOWN;  
 hlp->hypercode    = UN_KNOWN;  
 hlp->right_semaphore_code = UN_KNOWN; 
 hlp->left_semaphore_code  = UN_KNOWN; 
 hlp->uniform_code      = UN_KNOWN;     
 hlp->full_uniform_code = UN_KNOWN;     

 init_word_not_code(hlp); 
 /* --- Bordeaux 1995 : end --- */

 if (hlp->brex) 
   {
     freerex(hlp->lrex); 
     dispose(hlp->lrex);
   }

 if (hlp->stariscomp) 
   {
     freesf(hlp->sexp,TRUE,hlp->lmon->mno);
     dispose(hlp->sexp);
   }

 if (hlp->bdfa) 
   {
     freedfa(hlp->ldfa);
     dispose(hlp->ldfa);
   }
 
 if (hlp->bnfa) 
   {
     freenfa(hlp->lnfa);
     dispose(hlp->lnfa);
   }

 if (hlp->bmon) 
   {
     freemon(hlp->lmon); 
     dispose(hlp->lmon);
   }
 
 hlp->brex       = FALSE;
 hlp->stariscomp = FALSE;
 hlp->bdfa       = FALSE;
 hlp->bnfa       = FALSE;
 hlp->bmon       = FALSE;
 
 switch(hlp->inputis) 
   {
   case OREXINP: 
     freerex(hlp->orex);
     dispose(hlp->orex); 
     break;
   case ODFAINP: 
     freedfa(hlp->odfa);
     dispose(hlp->odfa); 
     break;
   case ONFAINP: 
     freenfa(hlp->onfa);
     dispose(hlp->onfa); 
     break;
   case OENFAINP:
     freenfa(hlp->oenfa);
     dispose(hlp->oenfa); 
     break;
   }
 
 hlp->inputis = 0;
 
 return hlp;
} /* newlang */

/******************************************************************/
void
movelang(a,b)
     int a,b;
/* move language langs[b] to langs[a] */
{ register language i,j;
  register int k;

  i = langs[a]; 
  j = langs[b];

  for(k = 0; k < NLEN; k++) 
    {
      i->name[k] = j->name[k];
      j->name[k] = '\0';
    }

  for(k = 0; k < REFLEN; k++) 
    {
      i->ref[k] = j->ref[k];
      j->ref[k] = '\0';
    }

  i->saved  = j->saved; 
  j->saved  = FALSE;
  i->sizeal = j->sizeal; j->sizeal = 0;

		
  i->empty = j->empty; 
  j->empty = UN_KNOWN;
  i->full = j->full; 
  j->full = UN_KNOWN;
  i->folu = j->folu;
  j->folu = UN_KNOWN;
  i->starfree = j->starfree;
  j->starfree = UN_KNOWN;
  i->localtest = j->localtest;
  j->localtest = UN_KNOWN;
  i->definite = j->definite;
  j->definite = UN_KNOWN;
  i->revdefinite = j->revdefinite;
  j->revdefinite = UN_KNOWN;
  i->gendefinite = j->gendefinite;
  j->gendefinite = UN_KNOWN;
  i->dotdepth1 = j->dotdepth1;
  j->dotdepth1 = UN_KNOWN;
  i->piecetest = j->piecetest;
  j->piecetest = UN_KNOWN;
  i->nilpotent = j->nilpotent;
  j->nilpotent = UN_KNOWN;
/* --- Bordeaux : 1995 : start --- */
  i->code         = j->code;        
  j->code         = UN_KNOWN;       
  i->stack_code   = j->stack_code;  
  j->stack_code   = UN_KNOWN;       
  i->prefix_code  = j->prefix_code; 
  j->prefix_code  = UN_KNOWN;       
  i->suffix_code  = j->suffix_code; 
  j->suffix_code  = UN_KNOWN;       
  i->bifix_code   = j->bifix_code;  
  j->bifix_code   = UN_KNOWN;       
  i->p_infix_code = j->p_infix_code;
  j->p_infix_code = UN_KNOWN;       
  i->s_infix_code = j->s_infix_code;
  j->s_infix_code = UN_KNOWN;       
  i->infix_code   = j->infix_code;  
  j->infix_code   = UN_KNOWN;       
  i->outfix_code  = j->outfix_code; 
  j->outfix_code  = UN_KNOWN;       
  i->hypercode    = j->hypercode;   
  j->hypercode    = UN_KNOWN;       
  i->right_semaphore_code = j->right_semaphore_code;
  j->right_semaphore_code = UN_KNOWN;               
  i->left_semaphore_code  = j->left_semaphore_code; 
  j->left_semaphore_code  = UN_KNOWN;               
  i->uniform_code      = j->uniform_code;     
  j->uniform_code      = UN_KNOWN;            
  i->full_uniform_code = j->full_uniform_code;
  j->full_uniform_code = UN_KNOWN;            

  i->word_not_code = strdup(j->word_not_code);
  init_word_not_code(j);                      
  /* --- Bordeaux : 1995 : end --- */

  i->localdegree = j->localdegree; 
  j->localdegree = 0;

  if (i->brex) 
    {
      freerex(i->lrex); 
      dispose(i->lrex);
    }
  
  if (i->stariscomp) 
    {
      freesf(i->sexp,TRUE,i->lmon->mno);
      dispose(i->sexp);
    }
  
  if (i->bdfa) 
    {    
      freedfa(i->ldfa);
      dispose(i->ldfa);
    }
  
  if (i->bnfa) 
    {
      freenfa(i->lnfa); 
      dispose(i->lnfa);
    }
  
  if (i->bmon) 
    {
      freemon(i->lmon);
      dispose(i->lmon);
    }
  
  i->brex = j->brex; 
  j->brex = FALSE;
  i->stariscomp = j->stariscomp;
  j->stariscomp = FALSE;
  i->bdfa = j->bdfa;
  j->bdfa = FALSE;
  i->bnfa = j->bnfa; 
  j->bnfa = FALSE;
  i->bmon = j->bmon; 
  j->bmon = FALSE;
  
  switch(i->inputis) 
    {
    case OREXINP: 
      freerex(i->orex);
      dispose(i->orex); 
      break;
    case ODFAINP: 
      freedfa(i->odfa);
      dispose(i->odfa); 
      break;
    case ONFAINP: 
      freenfa(i->onfa);
      dispose(i->onfa); 
      break;
    case OENFAINP:
      freenfa(i->oenfa);
      dispose(i->oenfa); 
      break;
    }

  i->inputis = j->inputis; j->inputis = 0;
  
  i->lrex = j->lrex;
  i->sexp = j->sexp;
  i->ldfa = j->ldfa;
  i->lnfa = j->lnfa;
  i->lmon = j->lmon;
  i->orex = j->orex;
  i->odfa = j->odfa;
  i->onfa = j->onfa;
  i->oenfa = j->oenfa;
} /* movelang */

/******************************************************************/
ddelta
newddelta(s,q)
posint s,q;
/* allocate memory for the transition function of a deterministic automaton
 * s,q are the maximal letter and the max. state
 */
{ ddelta hlp;
  register posint i,j;
	hlp = (ddelta)(calloc(s+1,sizeof(trans)));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newddelta 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	for(i=1; i<=s; ) {
		hlp[i] = (trans)(calloc(q+1,sizeof(posint)));
		if (hlp[i++] == NULL) {
#ifdef DEBUG
			perror("newddelta 2"); 
#endif
			for(j=i-1; --j;) dispose(hlp[j]);
			dispose(hlp);
			buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
		}
	}
	return hlp;
} /* newddelta */

/******************************************************************/
static ndelta
newnndelta(s,q,s0)
posint s,q;
/* allocate memory for the transition relation of a nondeterministic automaton
 * [s0...s][0...q]
 * s, q are the maximal letter and the maximal state 
 * s0 first letter
 */
{ ndelta hlp;
  register posint k,j,i,l;
  hlp = (ndelta)(calloc(s+1,sizeof(ntrans)));
  if(hlp == NULL) 
    {
#ifdef DEBUG
     perror("newndelta 1"); 
#endif
     buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
    }
  for(i=s0; i<=s; i++) 
    {hlp[i] = (ntrans)(calloc(q+1,sizeof(qlist)));
     if(hlp[i] == NULL) 
       {
#ifdef DEBUG
	perror("newndelta 2"); 
#endif
	for(k=s0;k<i;k++) dispose(hlp[k]);
	dispose(hlp);
	buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
       }
     k = lastdelta(q);
     for(j=0; j<=q; j++) 
       {hlp[i][j] = (qlist)(calloc(k+1,sizeof(char)));
	if(hlp[i][j] == NULL) 
          {
#ifdef DEBUG
	perror("newndelta 3"); 
#endif
	   for(k=j; k--;) dispose(hlp[i][k]);
	   dispose(hlp[i]);
	   for(l=s0;l<i;l++)
             {for(k=q+1; k--;) dispose(hlp[l][k]);
	      dispose(hlp[l]);
             }
	   dispose(hlp);
	   buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	  }
       }
   }
   return hlp;
} /* newnndelta */

/******************************************************************/
ndelta
newendelta(s,q)
posint s,q;
/* allocate memory for the transition relation of a nondeterministic automaton
 * [0...s][0...q]
 * s, q are the maximal letter and the maximal state 
 */
{return(newnndelta(s,q,0));}

ndelta
newndelta(s,q)
/* allocate memory for the transition relation of a nondeterministic automaton
 * [1...s][0...q]
 * s, q are the maximal letter and the maximal state 
 */
{return(newnndelta(s,q,1));}

/******************************************************************/
string
newrexstr(strl)
posint strl;
{ string hlp;
	hlp = (string)calloc(strl+1, sizeof(char));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newrexstr 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	return hlp;
} /* newrexstr */

/******************************************************************/
mrkfin
newfinal(a)
posint a;
{ mrkfin hlp;
	hlp = (mrkfin)calloc(a+1, sizeof(boolx));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newfinal 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	return hlp;
} /* newfinal */

/******************************************************************/
dfa
newdfa()
{ dfa hlp;
	hlp = (dfa)calloc(1, sizeof(struct dfauto));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newdfa 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	return hlp;
} /* newdfa */

/******************************************************************/
nfa
newnfa()
{ nfa hlp;
	hlp = (nfa)calloc(1, sizeof(struct nfauto));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newnfa 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	return hlp;
} /* newnfa */

/******************************************************************/
regex
newrex()
{ regex hlp;
	hlp = (regex)calloc(1, sizeof(struct rex));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newrex 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	return hlp;
} /* newrex */

/******************************************************************/
monoid
newmon()
{ monoid hlp;
	hlp = (monoid)calloc(1, sizeof(struct mono));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newmon 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	return hlp;
} /* newmon */

/******************************************************************/
array
newar(a)
posint a;
{ array hlp;
	hlp = (array)calloc(a, sizeof(posint));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newar 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	return hlp;
} /* newar */

/******************************************************************/
boole *
newbarray(a)
posint a;
{ boole *hlp;
	hlp = (boole *)calloc(a, sizeof(boole));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newbarray 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	return hlp;
} /* newbarray */

/******************************************************************/
arrayofarray
newarray1(a)
posint a;
{ arrayofarray hlp;
	hlp = (arrayofarray)calloc(a,sizeof(array));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newarray 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	return hlp;
} /* newarray1 */

/******************************************************************/

d_class
newdclass()
{ d_class hlp;
	hlp = (d_class)calloc(1, sizeof(struct dclass));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newdclass 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	return hlp;
} /* newdclass */

/******************************************************************/
dstruct
newdstruct()
{ dstruct hlp;
	hlp = (dstruct)calloc(1, sizeof(struct d_struct));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newdstruct 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	return hlp;
} /* newdstruct */

/******************************************************************/
rstruct
newrstruct()
{ rstruct hlp;
	hlp = (rstruct)calloc(1, sizeof(struct r_struct));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newrstruct 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	return hlp;
} /* newrstruct */

/******************************************************************/
starfexp
newsfexp()
{ starfexp hlp;
	hlp = (starfexp)calloc(1, sizeof(struct sexp));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newsfexp 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	return hlp;
} /* newsfexp */

/******************************************************************/
darray
newdarray(a)
posint a;
{ darray hlp;
	hlp = (darray)calloc(a, sizeof(d_class));
	if (hlp == NULL) {
#ifdef DEBUG
		perror("newdarray 1"); 
#endif
		buffree(); freebuf(); clock_reset(); (void)longjmp(env,1);
	}
	return hlp;
} /* newdarray */
