/* monrel.c 3.0 (KIEL) 11.94 */

#include "ext.h"

/******************************************************************/
/* functions included:
 *   functions for mon2rel 
 *      static posint revlex()
 *      static boole inrel()
 *      static void  mon2rel()
 *   functions for output of relations
 *      static void preout() 
 *             void editrel()
 *             void prtrel()
 */
/******************************************************************/


#define newreltree()	(reltree)newbuf((posint)1,(posint)sizeof(struct reltrees))
typedef struct reltrees{posint left;     /* left side of relation = element of monoid */
			posint gen;      /* generator */
			struct reltrees *lsucc,*rsucc;
                        struct reltrees *next;
			} *reltree;

static monoid stamon;

/* static variables for the computation of relations */
static boole emptytree;
static reltree relroot;  /* root of ordered tree */
static reltree inprel;   /* pointer to the node in the tree where relation 
                            is inserted */

/* static variables for output of relations */
static  arrayofstring left,generator,right; /* relations */
static  array         lleft,lgenerator,lright; /* length of relations */
static int maxrel,maxleft; /* max size of a relation,left side of a relation */
/******************************************************************/
/******************************************************************/
static posint revlex(rel1,rel2)
/* a relation rel1 ua=v is greater than relation rel2 wb=x if
 * b^rev(w) is lexicographically greater than a^rev(u)
 *
 * rel1->left must be longer than rel2->left
 *
 * 1 if relation rel2 implies relation rel1  
 * 0 if  rel1 is greater rel2 
 * 2 if  rel2 is greater rel1 
 */
reltree rel1,rel2;	
{posint pre1,pre2;
 array last;  /* abbreviation */
 if(rel1->gen<rel2->gen) return(2);
 else if(rel1->gen>rel2->gen) return(0);
 pre1=rel1->left;
 pre2=rel2->left;
 last=stamon->lastletter;
 while((pre2)&&(last[pre1]==last[pre2])) 
    /* if pre2==0 then word is empty */
   {pre1=stamon->gensucc[pre1][0];
    pre2=stamon->gensucc[pre2][0];
   }
 if(!pre2) return(1);
 if(last[pre1]<last[pre2]) return(2);
 else                      return(0);
}

/*******************************************************************************/
static boole inrel(rel)
/* create new node if relation is new and return TRUE 
 * static variable inprel points to the new node 
 */
reltree rel;
{posint j;
 reltree search;
 if(emptytree) /* init tree */
   {relroot=newreltree();
    relroot->left=rel->left;
    relroot->gen=rel->gen;
    inprel=relroot;
    emptytree=FALSE;
    return(TRUE);
   }
 else
   {search=relroot;
    for(;;)
      {j=revlex(rel,search);
       if (j==1) return(FALSE); /* new relation is an implication of an old one */
       else if(j==0) /* search at left successor  */
         {if(search->lsucc!=NULL) search=search->lsucc; /* continue */
	  else    /* new relation */
	    {inprel=newreltree();
	     inprel->left=rel->left;
	     inprel->gen=rel->gen;
	     search->lsucc=inprel;
	     return(TRUE);
	    }
         }
       else /* search at right successor */
         {if(search->rsucc!=NULL) search=search->rsucc;  /* continue */
	  else   /* new relation */
	    {inprel=newreltree();
	     inprel->left=rel->left;
	     inprel->gen=rel->gen;
	     search->rsucc=inprel;
	     return(TRUE);      
	    }
         }
      }
   }
}

/*******************************************************************************/
void mon2rel(mon)
/* compute defining relations of monoid stamon
 *    i.e. equation of the form w1 = w2
 *    where w1 and w2 are representatives of the same monoidelelement
 *    and w1 is lexicographical greater than w2
 * possible relations are stored in list relfirst
 *   this list is computed in dfa2mon during the computation of the monoid
 * there are many relations which follow from other relations
 *   for example relations of the form uv.a = uw with v.a=w is also a relation
 *   relations like this are deleted
 * relations of the form zeroa=zero and azero=zero are deleted 
 * relations are ordered in tree 
 */ 
 monoid mon;
{boole new;
 posint i,gen,zero;
 posint countrel=0; /* count number of relations in list relfirst */
 arrayofarray succ=mon->gensucc;
 reltree relfirst,rellast; /* first and last relation in list of relations */
 reltree rel;
 stamon=mon;
 stamon->rs=newrstruct();
 emptytree=TRUE; /* emptytree = TRUE  iff  tree is empty */
 /* build tree of relations */
 rel=newreltree();
 /* insert zero-relation at the beginning of the list */
 zero=stamon->zero;
 if(zero!=stamon->mno)
   {countrel=1;
    rel->left=succ[zero][0];
    rel->gen=stamon->lastletter[zero];
    relfirst=rel;
    rellast=rel;
    rel=newreltree();
   }
 for(i=0;i<stamon->mno;i++)
   for(gen=1;gen<=stamon->gno;gen++)
     {new=FALSE;
      if(succ[succ[i][gen]][0]!=i) new=TRUE; /* pred(i*gen)!= i */
      else if(stamon->lastletter[succ[i][gen]] != gen) new=TRUE; /* i*gen=i*gen' with gen'< gen */ 
      if(new)
         {rel->gen=gen;
          rel->left=i;
          if(zero==succ[i][gen]) 
            {if((rel->left==zero)||(rel->gen==zero))new=FALSE; /*left=zero,gen=zero*/
             else if(revlex(rel,relfirst)==1)new=FALSE;        /*left*gen=gen'*zero*/
            }
          if(new)
            if(inrel(rel))
             {if(countrel) /* insert relation at the end */
               {rellast->next=inprel;
                rellast=rellast->next;
               }
              else
               {relfirst=inprel;
                rellast=relfirst;
               }
              countrel++;tick();
             }
          }
     } 
/* read out relations */
 stamon->rs->rno=countrel;
 stamon->rs->lside=newar(countrel);
 stamon->rs->rside=newar(countrel);
 for(i=0;i<countrel;i++)
   {stamon->rs->rside[i]=relfirst->gen;
    stamon->rs->lside[i]=relfirst->left;
    relfirst=relfirst->next;
   }
 stamon->relationcomputed=TRUE;
 freebuf();
}

/*****************************************************************************/
static posint preout(mon)
/* prepare output of relations
 * compute strings for rightside generator leftside
 * compute lengths of this strings 
 * compute maximal length of relations  
 */
monoid mon;
{posint length,length1;
 posint rno=mon->rs->rno;
 char *word;
 posint i;
 /* compute numer of trivial relations */
 for(i=1;i<=mon->sno;i++)
   if((mon->let2gen[i]==0)||(mon->generator[mon->let2gen[i]]!=i))rno++;
 /* memory */
 left=newarrayofstring(rno);
 generator=newarrayofstring(rno);
 right=newarrayofstring(rno);
 lleft=newarray(rno);
 lgenerator=newarray(rno);
 lright=newarray(rno);
 /* compute relations + maximal length of a leftside,relation */
 maxrel=0;maxleft=0;
 for(rno=0;rno<mon->rs->rno;rno++) /* for all relations */
   {(void)prword1(&word,TRUE,&length,mon->rs->lside[rno],mon,FALSE,FALSE);
    left[rno]=word;
    lleft[rno]=length;
    (void)prword1(&word,TRUE,&length1,mon->rs->rside[rno],mon,FALSE,FALSE);
    generator[rno]=word;
    lgenerator[rno]=length1;
    length+=length1;
    if(maxleft<length)maxleft=length;
    (void)prword1(&word,TRUE,&length1,mon->gensucc[mon->rs->lside[rno]][mon->rs->rside[rno]],mon,FALSE,TRUE);
    right[rno]=word;
    lright[rno]=length1;
    length+=length1;
    if(maxrel<length)maxrel=length;
   }  
 /* trivial relations appended */
  for(i=1;i<=mon->sno;i++)
   if((mon->let2gen[i]==0)||(mon->generator[mon->let2gen[i]]!=i))
      {  
       left[rno]=newstring(1);
       left[rno][0]='\0';
       lleft[rno]=0;
       if (mon->sno > 27)
              {lgenerator[rno]=1+strlen(pi2a(i));
               generator[rno]=newstring(lgenerator[rno]+1);
               (void)strcat(generator[rno],"a");
               (void)strcat(generator[rno],pi2a(i));
              }  
       else
              {lgenerator[rno]=1;
               generator[rno]=newstring(2);
               generator[rno][0]=itoc[i];
               generator[rno][1]='\0';
              }  

       if(mon->let2gen[i]) /* relation a=b */
          (void)prword1(&word,TRUE,&length1,mon->generator[mon->let2gen[i]],mon,FALSE,FALSE);
       else                  /* relation a=1 */
          (void)prword1(&word,TRUE,&length1,0,mon,FALSE,TRUE);
       right[rno]=word;
       lright[rno]=length1;
       rno++;
      }  
 return(rno);
}

/*****************************************************************************/
void editrel(mon)
monoid mon;
{
 int inforow=5; /* first row */
 int firstcol=5; /* first columne */
 int equalcol;
 int diff;
 int rowco,colco,rowco1,colco1;
 int line,line1;
 posint rno;
 boole  gt1line; /* relations are longer than one line */
 posint r=preout(mon); /* add new relations */
 /* set output constants */
 maxrel+=(3+firstcol); /* equality sign + free space */
 gt1line=(maxrel>(COLS-firstcol)); /* a relation needs more than one line */
 equalcol=(gt1line)? 10 : maxleft+firstcol+1; /* columne for equality sign */
 diff=(equalcol< 9 )? equalcol-(firstcol+1) : 8; /* start of the title */
 rowco=inforow;
 clear();
 if(gt1line) /* long relations */
     for(rno=0;rno<r;rno++)
       {
	line=needlines(firstcol,COLS,(char *)NULL,lleft[rno]+lgenerator[rno],TRUE,firstcol+2);
	line1=needlines(firstcol,COLS,(char *)NULL,lright[rno],TRUE,firstcol+2);
	if((rowco+line+line1)>=LASTROW) /* new page */
	  {
	   if(gt1line) maketitle(TRUE,"R E L A T I O N S");
	   else mvaddstr(TITLEROW,equalcol-diff,"R E L A T I O N S");
	   mvaddstr(LASTROW,5,"press any key to continue");
	   refresh(); (void)getch(); clear();
	   rowco=inforow;
	  }
        prtstr(rowco,firstcol,COLS-1,left[rno],1,firstcol+2,LINES-1,FALSE,TRUE,&rowco1,&colco1);
	if(colco1==COLS){rowco1++;colco1=firstcol;} 
        prtstr(rowco1,colco1,COLS-1,generator[rno],1,firstcol+2,LINES-1,FALSE,TRUE,&rowco,&colco);
	rowco++; 
	mvaddch(rowco,equalcol,'=');
	rowco++;
	prtstr(rowco,firstcol,COLS-1,right[rno],1,firstcol+2,LINES-1,FALSE,TRUE,&rowco1,&colco1);
	rowco=2+rowco1;
      }
 else /* relation in one line */
     for(rno=0;rno<r;rno++)
       {
	mvaddstr(rowco,firstcol,left[rno]);
	addstr(generator[rno]);
	mvaddch(rowco,equalcol,'=');
	mvaddstr(rowco,equalcol+2,right[rno]);
	rowco++;
	if(rowco==LASTROW)
	   {
	    if(gt1line) maketitle(TRUE,"R E L A T I O N S");
	    else mvaddstr(TITLEROW,equalcol-diff,"R E L A T I O N S");
	    mvaddstr(LASTROW,5,"press any key to continue");
	    refresh(); (void)getch(); clear();
	    rowco=inforow;
	    }
       }
if(gt1line) maketitle(TRUE,"R E L A T I O N S");
 else mvaddstr(TITLEROW,equalcol-diff,"R E L A T I O N S");
 mvaddstr(LASTROW,5,"press any key to continue");
 refresh();
 (void)getch();
 freebuf();
} 

/*****************************************************************************/
void prtrel(mon)
monoid mon;
{
 /* 
  *       leftside = (left page) | (right page)= right side 
  *            (fails if leftside needs more then one page)
  */
 /* output constants */
 int pgrowlast=plines-2,pgrowfirst=1; /* first,last row used */
 int pgcollast=pcols-3,pgcolfirst=4; /* first,last col */
 int equalleft;
 int equalright=pgcolfirst-2; /* cols for equalitysign */
 int rowco,colco,rowco1,colco1;
 int line;
 posint rno,lno;
 boole  twopages; /* relations are longer than one line */
 posint startpage,count;
 array  onside;
 /* set output constants */
 preout(mon);
 maxrel+=(3+pgcolfirst); /* equality sign + free space on left side */
 twopages=(maxrel>(pgcollast+1-pgcolfirst)); /* relation needs more than one line */
 equalleft=(twopages)? pgcollast+1 : maxleft+pgcolfirst+2; /* columne for equality sign */
 rowco=pgrowfirst+1;
 pgmvaddstr(pgrowfirst,0, "R E L A T I O N S");
 rowco=pgrowfirst+2; /* one empty line */
 if(twopages) /* long relations */
   {/* get memory for onside : number of lines on page */
    onside=newarray(pgrowlast-pgrowfirst+1); 
    startpage=0;
    count=0;
    /* special comment on firstside explaining the output format */
    for(rno=0;rno<=mon->rs->rno;rno++)
       {if(rno<mon->rs->rno)
	 line=needlines(pgcolfirst,pgcollast,(char *)NULL,lleft[rno]+lgenerator[rno],TRUE,pgcolfirst+2);
	if((rno==mon->rs->rno)||((rowco+line-1)>pgrowlast))
	    /* print left page if 1. right page is full 
	     *                    2. last relation is printed
	     */
	  {/* first move page left   
	    * print all left sides for relations startpage to startpage+count-1
	    *   starting at rows stored in onside
	    * move page down 
	    */
           pgbreak();
	   gohorizontal();
           for(lno=0;lno<count;lno++)
	      {rowco=onside[lno];
	       pgmvaddch(rowco,equalright,'=');
	       prtstr(rowco,equalright+2,pgcollast-1,right[lno+startpage],1,pgcolfirst+2,plines-1,TRUE,FALSE); 
              }
	   /* actualize variables only if not last relation */
	   if(rno<mon->rs->rno)
	      {rowco=pgrowfirst; startpage=rno;
	       count=0;
	       pgbreak(); govertical();
	      }
	  } 
	/* print leftside of relation rno if not rno == mon->rs->rno */
        if(rno<mon->rs->rno)
	   {onside[count]=rowco;
	    pgmvaddch(rowco,equalleft,'=');
            count++;
            prtstr(rowco,pgcolfirst,pgcollast,left[rno],1,pgcolfirst+2,plines,TRUE,TRUE,&rowco1,&colco1);
	    if(colco1==(pgcollast+1)){rowco1++;colco1=pgcolfirst;} 
            prtstr(rowco1,colco1,pgcollast,generator[rno],1,pgcolfirst+2,plines,TRUE,TRUE,&rowco,&colco);
	    rowco++; 
           }
       }
    }
 else /* relation in one line */
    for(rno=0;rno<mon->rs->rno;rno++)
       {
	pgmvaddstr(rowco,pgcolfirst,left[rno]);
	pgaddstr(generator[rno]);
	pgmvaddch(rowco,equalleft ,'=');
	pgmvaddstr(rowco,equalleft+2,right[rno]);
	rowco++;
	if((rowco==pgrowlast)&&((rno!=mon->rs->rno-1)))
	   {
	    pgbreak();
	    govertical();
	    rowco=pgrowfirst;
	    }
       }
 pgbreak();
 freebuf();
} 
